@@ -6,11 +6,6 @@ use openpgp::parse::Parse;
 
 use crate::common::encrypt;
 
-lazy_static::lazy_static! {
-    static ref ZEROS_1_MB: Vec<u8> = vec![0; 1024 * 1024];
-    static ref ZEROS_10_MB: Vec<u8> = vec![0; 10 * 1024 * 1024];
-}
-
 pub fn encrypt_to_donald_sign_by_ivanka(bytes: &[u8]) {
     let sender = Cert::from_bytes(
         &include_bytes!("../tests/data/keys/ivanka-private.gpg")[..],
@@ -26,10 +21,7 @@ pub fn encrypt_to_donald_sign_by_ivanka(bytes: &[u8]) {
 fn bench_encrypt_sign(c: &mut Criterion) {
     let mut group = c.benchmark_group("encrypt and sign message");
 
-    // Encrypt a very short, medium and very long message.
-    let messages = &[b"Hello world.", &ZEROS_1_MB[..], &ZEROS_10_MB[..]];
-
-    for message in messages {
+    for message in encrypt::messages() {
         group.throughput(Throughput::Bytes(message.len() as u64));
         group.bench_with_input(
             BenchmarkId::new("encrypt and sign", message.len()),
