@@ -29,20 +29,18 @@ fn main() -> openpgp::Result<()> {
             .context("Failed to read key")?;
         let mut n = 0;
 
-        for key in tsk
+        for skb in tsk
             .keys().with_policy(p, None).alive().revoked(false).for_signing().secret()
-            .map(|ka| ka.key())
         {
             keys.push({
-                let mut key = key.clone();
+                let mut key = skb.key().clone();
                 if key.secret().is_encrypted() {
                     let password = rpassword::prompt_password(format!(
                         "Please enter password to decrypt {}/{}: ",
                         tsk, key
                     ))?;
-                    let algo = key.pk_algo();
                     key.secret_mut()
-                        .decrypt_in_place(algo, &password.into())
+                        .decrypt_in_place(skb.key(), &password.into())
                         .context("decryption failed")?;
                 }
                 n += 1;
@@ -68,9 +66,9 @@ fn main() -> openpgp::Result<()> {
 
     // Now, create a signer that emits the detached signature(s).
     let mut signer =
-        Signer::new(message, keys.pop().context("No key for signing")?);
+        Signer::new(message, keys.pop().context("No key for signing")?)?;
     for s in keys {
-        signer = signer.add_signer(s);
+        signer = signer.add_signer(s)?;
     }
     let mut message =
         signer.detached().build().context("Failed to create signer")?;
