@@ -102,8 +102,8 @@ pub struct KeyAmalgamationIter<'a, P, R>
     // secret.
     unencrypted_secret: Option<bool>,
 
-    // Only return keys in this set.
-    key_handles: Option<Vec<KeyHandle>>,
+    /// Only return keys in this set.
+    key_handles: Vec<KeyHandle>,
 
     // If not None, filters by whether we support the key's asymmetric
     // algorithm.
@@ -191,15 +191,13 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
 
             t!("Considering key: {:?}", ka.key());
 
-            if let Some(key_handles) = self.key_handles.as_ref() {
-                if !key_handles
-                    .iter()
+            if ! self.key_handles.is_empty()
+                && ! self.key_handles.iter()
                     .any(|h| h.aliases(ka.key().key_handle()))
-                {
-                    t!("{} is not one of the keys that we are looking for ({:?})",
-                       ka.key().fingerprint(), self.key_handles);
-                    continue;
-                }
+            {
+                t!("{} is not one of the keys that we are looking for ({:?})",
+                   ka.key().fingerprint(), self.key_handles);
+                continue;
             }
 
             if let Some(want_supported) = self.supported {
@@ -243,7 +241,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
             // The filters.
             encrypted_secret: None,
             unencrypted_secret: None,
-            key_handles: None,
+            key_handles: Default::default(),
             supported: None,
 
             _p: std::marker::PhantomData,
@@ -384,9 +382,9 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # use openpgp::cert::prelude::*;
     /// # fn main() -> Result<()> {
     /// #     let (cert, _) =
-    /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #         CertBuilder::general_purpose(Some("alice@example.org"))
     /// #         .generate()?;
-    /// # let key_handle = cert.primary_key().key_handle();
+    /// # let key_handle = cert.primary_key().key().key_handle();
     /// # let mut i = 0;
     /// for ka in cert.keys().key_handle(key_handle) {
     ///     // Use it.
@@ -403,16 +401,16 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     pub fn key_handle<H>(mut self, h: H) -> Self
         where H: Into<KeyHandle>
     {
-        if self.key_handles.is_none() {
-            self.key_handles = Some(Vec::new());
-        }
-        self.key_handles.as_mut().unwrap().push(h.into());
+        self.key_handles.push(h.into());
         self
     }
 
     /// Changes the iterator to only return a key if it matches one of
     /// the specified `KeyHandle`s.
     ///
+    /// If the given `handles` iterator is empty, the set of returned
+    /// keys is not constrained.
+    ///
     /// This function is cumulative.  If you call this function (or
     /// [`key_handle`]) multiple times, then the iterator returns a key
     /// if it matches *any* of the specified [`KeyHandle`s].
@@ -428,11 +426,11 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # use openpgp::cert::prelude::*;
     /// # fn main() -> Result<()> {
     /// #     let (cert, _) =
-    /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #         CertBuilder::general_purpose(Some("alice@example.org"))
     /// #         .generate()?;
-    /// # let key_handles = &[cert.primary_key().key_handle()][..];
+    /// # let key_handles = &[cert.primary_key().key().key_handle()][..];
     /// # let mut i = 0;
-    /// for ka in cert.keys().key_handles(key_handles.iter()) {
+    /// for ka in cert.keys().key_handles(key_handles) {
     ///     // Use it.
     /// #   i += 1;
     /// }
@@ -444,14 +442,16 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// [`KeyHandle`s]: super::super::super::KeyHandle
     /// [`key_handle`]: KeyAmalgamationIter::key_handle()
     /// [`KeyHandle::aliases`]: super::super::super::KeyHandle::aliases()
-    pub fn key_handles<'b>(mut self, h: impl Iterator<Item=&'b KeyHandle>)
-        -> Self
-        where 'a: 'b
+    pub fn key_handles<H, K>(mut self, handles: H) -> Self
+    where
+        H: IntoIterator<Item=K>,
+        K: Borrow<KeyHandle>,
     {
-        if self.key_handles.is_none() {
-            self.key_handles = Some(Vec::new());
-        }
-        self.key_handles.as_mut().unwrap().extend(h.cloned());
+        let mut handles = handles.into_iter()
+            .map(|h| h.borrow().clone())
+            .collect::<Vec<_>>();
+
+        self.key_handles.append(&mut handles);
         self
     }
 
@@ -469,7 +469,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::cert::prelude::*;
     /// #     let (cert, _) =
-    /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #         CertBuilder::general_purpose(Some("alice@example.org"))
     /// #         .generate()?;
     /// # let mut i = 0;
     /// for ka in cert.keys().supported() {
@@ -554,7 +554,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// let p = &StandardPolicy::new();
     ///
     /// #     let (cert, _) =
-    /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #         CertBuilder::general_purpose(Some("alice@example.org"))
     /// #         .generate()?;
     /// #     let fpr = cert.fingerprint();
     /// // Iterate over all valid User Attributes.
@@ -608,7 +608,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
 /// A `ValidKeyAmalgamationIter` also provides additional
 /// filters based on information available in the `Key`s' binding
 /// signatures.  For instance, [`ValidKeyAmalgamationIter::revoked`]
-/// filters the returned `Key`s by whether or not they are revoked.
+/// filters the returned `Key`s by whether they are revoked.
 /// And, [`ValidKeyAmalgamationIter::alive`] changes the iterator to
 /// only return `Key`s that are live.
 ///
@@ -633,7 +633,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
 /// let p = &StandardPolicy::new();
 ///
 /// #     let (cert, _) =
-/// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
+/// #         CertBuilder::general_purpose(Some("alice@example.org"))
 /// #         .generate()?;
 /// # let mut i = 0;
 /// // The certificate *and* keys need to be valid.
@@ -697,8 +697,8 @@ pub struct ValidKeyAmalgamationIter<'a, P, R>
     // secret.
     unencrypted_secret: Option<bool>,
 
-    // Only return keys in this set.
-    key_handles: Option<Vec<KeyHandle>>,
+    /// Only return keys in this set.
+    key_handles: Vec<KeyHandle>,
 
     // If not None, filters by whether we support the key's asymmetric
     // algorithm.
@@ -828,15 +828,13 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
             let key = ka.key();
             t!("Considering key: {:?}", key);
 
-            if let Some(key_handles) = self.key_handles.as_ref() {
-                if !key_handles
-                    .iter()
+            if ! self.key_handles.is_empty()
+                && ! self.key_handles.iter()
                     .any(|h| h.aliases(key.key_handle()))
-                {
-                    t!("{} is not one of the keys that we are looking for ({:?})",
-                       key.key_handle(), self.key_handles);
-                    continue;
-                }
+            {
+                t!("{} is not one of the keys that we are looking for ({:?})",
+                   key.key_handle(), self.key_handles);
+                continue;
             }
 
             if let Some(want_supported) = self.supported {
@@ -908,7 +906,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// `for_signing`, etc.) multiple times, the *union* of
     /// the values is used.
     ///
-    /// Note: [Section 12.1 of RFC 4880] says that the primary key is
+    /// Note: [Section 10.1 of RFC 9580] says that the primary key is
     /// certification capable independent of the `Key Flags`
     /// subpacket:
     ///
@@ -959,7 +957,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Ok(()) }
     /// ```
     ///
-    ///   [Section 12.1 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.2.3.21
+    ///   [Section 10.1 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.29
     ///   [`ValidKeyAmalgamation::key_flags`]: ValidKeyAmalgamation::key_flags()
     pub fn key_flags<F>(mut self, flags: F) -> Self
         where F: Borrow<KeyFlags>
@@ -979,7 +977,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// `for_signing`, etc.) multiple times, the *union* of
     /// the values is used.
     ///
-    /// Note: [Section 12.1 of RFC 4880] says that the primary key is
+    /// Note: [Section 10.1 of RFC 9580] says that the primary key is
     /// certification capable independent of the `Key Flags`
     /// subpacket:
     ///
@@ -1028,7 +1026,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// ```
     ///
     ///   [`ValidKeyAmalgamation::for_certification`]: ValidKeyAmalgamation::for_certification()
-    ///   [Section 12.1 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.2.3.21
+    ///   [Section 10.1 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.29
     ///   [`ValidKeyAmalgamation::key_flags`]: ValidKeyAmalgamation::key_flags()
     pub fn for_certification(self) -> Self {
         self.key_flags(KeyFlags::empty().set_certification())
@@ -1302,7 +1300,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     ///
     /// # fn main() -> Result<()> {
     /// #     let (cert, _) =
-    /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #         CertBuilder::general_purpose(Some("alice@example.org"))
     /// #         .generate()?;
     /// let p = &StandardPolicy::new();
     ///
@@ -1318,7 +1316,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     ///             RevocationStatus::CouldBe(_) =>
     ///                 // There is a designated revoker that we
     ///                 // could check, but don't (or can't).  To
-    ///                 // avoid a denial of service attack arising from
+    ///                 // avoid a denial-of-service attack arising from
     ///                 // fake revocations, we assume that the key has
     ///                 // not been revoked and return it.
     ///                 true,
@@ -1512,9 +1510,9 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// let p = &StandardPolicy::new();
     ///
     /// #     let (cert, _) =
-    /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #         CertBuilder::general_purpose(Some("alice@example.org"))
     /// #         .generate()?;
-    /// # let key_handle = cert.primary_key().key_handle();
+    /// # let key_handle = cert.primary_key().key().key_handle();
     /// # let mut i = 0;
     /// for ka in cert.keys().with_policy(p, None).key_handle(key_handle) {
     ///     // Use it.
@@ -1531,16 +1529,16 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     pub fn key_handle<H>(mut self, h: H) -> Self
         where H: Into<KeyHandle>
     {
-        if self.key_handles.is_none() {
-            self.key_handles = Some(Vec::new());
-        }
-        self.key_handles.as_mut().unwrap().push(h.into());
+        self.key_handles.push(h.into());
         self
     }
 
     /// Changes the iterator to only return a key if it matches one of
     /// the specified `KeyHandle`s.
     ///
+    /// If the given `handles` iterator is empty, the set of returned
+    /// keys is not constrained.
+    ///
     /// This function is cumulative.  If you call this function (or
     /// [`key_handle`]) multiple times, then the iterator returns a key
     /// if it matches *any* of the specified [`KeyHandle`s].
@@ -1560,11 +1558,11 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// let p = &StandardPolicy::new();
     ///
     /// #     let (cert, _) =
-    /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #         CertBuilder::general_purpose(Some("alice@example.org"))
     /// #         .generate()?;
-    /// # let key_handles = &[cert.primary_key().key_handle()][..];
+    /// # let key_handles = &[cert.primary_key().key().key_handle()][..];
     /// # let mut i = 0;
-    /// for ka in cert.keys().with_policy(p, None).key_handles(key_handles.iter()) {
+    /// for ka in cert.keys().with_policy(p, None).key_handles(key_handles) {
     ///     // Use it.
     /// #   i += 1;
     /// }
@@ -1574,16 +1572,18 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// ```
     ///
     /// [`KeyHandle`s]: super::super::super::KeyHandle
-    /// [`key_handle`]: ValidKeyAmalgamationIter::key_handle()
+    /// [`key_handle`]: KeyAmalgamationIter::key_handle()
     /// [`KeyHandle::aliases`]: super::super::super::KeyHandle::aliases()
-    pub fn key_handles<'b>(mut self, h: impl Iterator<Item=&'b KeyHandle>)
-        -> Self
-        where 'a: 'b
+    pub fn key_handles<H, K>(mut self, handles: H) -> Self
+    where
+        H: IntoIterator<Item=K>,
+        K: Borrow<KeyHandle>,
     {
-        if self.key_handles.is_none() {
-            self.key_handles = Some(Vec::new());
-        }
-        self.key_handles.as_mut().unwrap().extend(h.cloned());
+        let mut handles = handles.into_iter()
+            .map(|h| h.borrow().clone())
+            .collect::<Vec<_>>();
+
+        self.key_handles.append(&mut handles);
         self
     }
 
@@ -1601,7 +1601,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::cert::prelude::*;
     /// #     let (cert, _) =
-    /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #         CertBuilder::general_purpose(Some("alice@example.org"))
     /// #         .generate()?;
     /// # let mut i = 0;
     /// use openpgp::policy::StandardPolicy;
@@ -1838,6 +1838,7 @@ mod test {
     }
 
     #[test]
+    #[allow(deprecated)]
     fn select_supported() -> crate::Result<()> {
         use crate::types::PublicKeyAlgorithm;
         if ! PublicKeyAlgorithm::DSA.is_supported()
@@ -1850,7 +1851,7 @@ mod test {
             Cert::from_bytes(crate::tests::key("dsa2048-elgamal3072.pgp"))?;
         assert_eq!(cert.keys().count(), 2);
         assert_eq!(cert.keys().supported().count(), 1);
-        let p = &crate::policy::NullPolicy::new();
+        let p = unsafe { &crate::policy::NullPolicy::new() };
         assert_eq!(cert.keys().with_policy(p, None).count(), 2);
         assert_eq!(cert.keys().with_policy(p, None).supported().count(), 1);
         Ok(())
