@@ -42,7 +42,7 @@
 //! signatures are used to provide information about the whole
 //! certificate.  For instance, they can include the default `Key`
 //! expiry time.  This is used if a subkey's binding signature doesn't
-//! include a expiry.
+//! include an expiry.
 //!
 //! Self-revocations are revocation certificates issued by the key
 //! certificate holder.
@@ -73,10 +73,10 @@
 //! [`UserAttribute`]: crate::packet::user_attribute
 //! [`Unknown`]: crate::packet::Unknown
 //! [`Tag`]: crate::packet::Tag
-//! [designated revoker]: https://tools.ietf.org/html/rfc4880#section-5.2.3.15
+//! [designated revoker]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.23
 //! [`ComponentAmalgamation`]: super::amalgamation
 //! [`KeyAmalgamation`]: super::amalgamation::key::KeyAmalgamation
-//! [complicated semantics]: https://tools.ietf.org/html/rfc4880#section-5.2.3.3
+//! [complicated semantics]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.10
 
 use std::time;
 use std::cmp::{self, Ordering};
@@ -175,15 +175,6 @@ pub type UserAttributeBundle = ComponentBundle<UserAttribute>;
 pub type UnknownBundle = ComponentBundle<Unknown>;
 
 
-impl<C> Deref for ComponentBundle<C>
-{
-    type Target = C;
-
-    fn deref(&self) -> &Self::Target {
-        &self.component
-    }
-}
-
 impl<C> ComponentBundle<C> {
     /// Creates a new component.
     ///
@@ -218,11 +209,11 @@ impl<C> ComponentBundle<C> {
     /// #
     /// # fn main() -> openpgp::Result<()> {
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// // Display some information about any unknown components.
     /// for u in cert.unknowns() {
-    ///     eprintln!(" - {:?}", u.component());
+    ///     eprintln!(" - {:?}", u.bundle().component());
     /// }
     /// # Ok(()) }
     /// ```
@@ -231,7 +222,7 @@ impl<C> ComponentBundle<C> {
     }
 
     /// Returns a mutable reference to the component.
-    fn component_mut(&mut self) -> &mut C {
+    pub(crate) fn component_mut(&mut self) -> &mut C {
         &mut self.component
     }
 
@@ -254,12 +245,12 @@ impl<C> ComponentBundle<C> {
     /// let p = &StandardPolicy::new();
     ///
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// // Display information about each User ID's current active
     /// // binding signature (the `time` parameter is `None`), if any.
     /// for ua in cert.userids() {
-    ///     eprintln!("{:?}", ua.binding_signature(p, None));
+    ///     eprintln!("{:?}", ua.bundle().binding_signature(p, None));
     /// }
     /// # Ok(()) }
     /// ```
@@ -464,27 +455,20 @@ impl<C> ComponentBundle<C> {
     /// let p = &StandardPolicy::new();
     ///
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// for (i, ka) in cert.keys().enumerate() {
     ///     eprintln!("Key #{} ({}) has {:?} self signatures",
-    ///               i, ka.fingerprint(),
-    ///               ka.bundle().self_signatures2().count());
+    ///               i, ka.key().fingerprint(),
+    ///               ka.bundle().self_signatures().count());
     /// }
     /// # Ok(()) }
     /// ```
-    // XXXv2: Rename this function.
-    pub fn self_signatures2(&self)
-                            -> impl Iterator<Item=&Signature> + Send + Sync {
+    pub fn self_signatures(&self)
+                           -> impl Iterator<Item=&Signature> + Send + Sync {
         self.self_signatures.iter_verified(self.backsig_signer.as_ref())
     }
 
-    /// Returns the component's self-signatures.
-    #[deprecated(note = "Use self_signatures2 instead.")]
-    pub fn self_signatures(&self) -> &[Signature] {
-        &self.self_signatures.slice_verified(self.backsig_signer.as_ref())
-    }
-
     /// Returns the component's third-party certifications.
     ///
     /// The signatures are *not* validated.  They are sorted by their
@@ -501,27 +485,20 @@ impl<C> ComponentBundle<C> {
     /// let p = &StandardPolicy::new();
     ///
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// for ua in cert.userids() {
     ///     eprintln!("User ID {} has {:?} unverified, third-party certifications",
     ///               String::from_utf8_lossy(ua.userid().value()),
-    ///               ua.bundle().certifications2().count());
+    ///               ua.bundle().certifications().count());
     /// }
     /// # Ok(()) }
     /// ```
-    // XXXv2: Rename this function.
-    pub fn certifications2(&self)
-                           -> impl Iterator<Item=&Signature> + Send + Sync {
+    pub fn certifications(&self)
+                          -> impl Iterator<Item=&Signature> + Send + Sync {
         self.certifications.iter()
     }
 
-    /// Returns the component's third-party certifications.
-    #[deprecated(note = "Use certifications2 instead.")]
-    pub fn certifications(&self) -> &[Signature] {
-        &self.certifications
-    }
-
     /// Returns the component's revocations that were issued by the
     /// certificate holder.
     ///
@@ -539,28 +516,20 @@ impl<C> ComponentBundle<C> {
     /// let p = &StandardPolicy::new();
     ///
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// for u in cert.userids() {
     ///     eprintln!("User ID {} has {:?} revocation certificates.",
     ///               String::from_utf8_lossy(u.userid().value()),
-    ///               u.bundle().self_revocations2().count());
+    ///               u.bundle().self_revocations().count());
     /// }
     /// # Ok(()) }
     /// ```
-    // XXXv2: Rename this function.
-    pub fn self_revocations2(&self)
-                             -> impl Iterator<Item=&Signature> + Send + Sync {
+    pub fn self_revocations(&self)
+                            -> impl Iterator<Item=&Signature> + Send + Sync {
         self.self_revocations.iter_verified(self.backsig_signer.as_ref())
     }
 
-    /// Returns the component's revocations that were issued by the
-    /// certificate holder.
-    #[deprecated(note = "Use self_revocations2 instead.")]
-    pub fn self_revocations(&self) -> &[Signature] {
-        &self.self_revocations.slice_verified(self.backsig_signer.as_ref())
-    }
-
     /// Returns the component's revocations that were issued by other
     /// certificates.
     ///
@@ -578,41 +547,35 @@ impl<C> ComponentBundle<C> {
     /// let p = &StandardPolicy::new();
     ///
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// for u in cert.userids() {
     ///     eprintln!("User ID {} has {:?} unverified, third-party revocation certificates.",
     ///               String::from_utf8_lossy(u.userid().value()),
-    ///               u.bundle().other_revocations2().count());
+    ///               u.bundle().other_revocations().count());
     /// }
     /// # Ok(()) }
     /// ```
-    // XXXv2: Rename this function.
-    pub fn other_revocations2(&self)
-                              -> impl Iterator<Item=&Signature> + Send + Sync {
+    pub fn other_revocations(&self)
+                             -> impl Iterator<Item=&Signature> + Send + Sync {
         self.other_revocations.iter()
     }
 
-    /// Returns the component's revocations that were issued by other
-    /// certificates.
-    #[deprecated(note = "Use other_revocations2 instead.")]
-    pub fn other_revocations(&self) -> &[Signature] {
-        &self.other_revocations
-    }
-
-    /// Returns all of the component's Attestation Key Signatures.
+    /// Returns all the component's Certification Approval Key
+    /// Signatures.
     ///
     /// This feature is [experimental](crate#experimental-features).
     ///
     /// The signatures are validated, and they are sorted by their
     /// creation time, most recent first.
     ///
-    /// A certificate owner can use Attestation Key Signatures to
-    /// attest to third party certifications.  Currently, only userid
-    /// and user attribute certifications can be attested.  See
-    /// [Section 5.2.3.30 of RFC 4880bis] for details.
+    /// A certificate owner can use Certification Approval Key
+    /// Signatures to approve of third party certifications.
+    /// Currently, only user ID and user attribute certifications can
+    /// be approved.  See [Approved Certifications subpacket] for
+    /// details.
     ///
-    ///   [Section 5.2.3.30 of RFC 4880bis]: https://tools.ietf.org/html/draft-ietf-openpgp-rfc4880bis-10.html#section-5.2.3.30
+    ///   [Approved Certifications subpacket]: https://www.ietf.org/archive/id/draft-dkg-openpgp-1pa3pc-02.html#approved-certifications-subpacket
     ///
     /// # Examples
     ///
@@ -624,27 +587,27 @@ impl<C> ComponentBundle<C> {
     /// let p = &StandardPolicy::new();
     ///
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// for (i, uid) in cert.userids().enumerate() {
-    ///     eprintln!("UserID #{} ({:?}) has {:?} attestation key signatures",
-    ///               i, uid.email(),
-    ///               uid.attestations().count());
+    ///     eprintln!("UserID #{} ({:?}) has {:?} certification approval key signatures",
+    ///               i, uid.userid().email(),
+    ///               uid.bundle().approvals().count());
     /// }
     /// # Ok(()) }
     /// ```
-    pub fn attestations(&self)
-                      -> impl Iterator<Item = &Signature> + Send + Sync
+    pub fn approvals(&self)
+                     -> impl Iterator<Item = &Signature> + Send + Sync
     {
         self.attestations.iter_verified(None)
     }
 
-    /// Returns all of the component's signatures.
+    /// Returns all the component's signatures.
     ///
     /// Only the self-signatures are validated.  The signatures are
     /// sorted first by type, then by creation time.  The self
     /// revocations come first, then the self signatures,
-    /// then any key attestation signatures,
+    /// then any certification approval key signatures,
     /// certifications, and third-party revocations coming last.  This
     /// function may return additional types of signatures that could
     /// be associated to this component.
@@ -660,23 +623,31 @@ impl<C> ComponentBundle<C> {
     /// let p = &StandardPolicy::new();
     ///
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// for (i, ka) in cert.keys().enumerate() {
     ///     eprintln!("Key #{} ({}) has {:?} signatures",
-    ///               i, ka.fingerprint(),
-    ///               ka.signatures().count());
+    ///               i, ka.key().fingerprint(),
+    ///               ka.bundle().signatures().count());
     /// }
     /// # Ok(()) }
     /// ```
     pub fn signatures(&self)
                       -> impl Iterator<Item = &Signature> + Send + Sync
     {
-        self.self_revocations2()
-            .chain(self.self_signatures2())
-            .chain(self.attestations())
-            .chain(self.certifications2())
-            .chain(self.other_revocations2())
+        self.self_revocations()
+            .chain(self.self_signatures())
+            .chain(self.approvals())
+            .chain(self.certifications())
+            .chain(self.other_revocations())
+    }
+
+    /// Returns all the bundles' bad signatures.
+    pub(crate) fn bad_signatures(&self)
+        -> impl Iterator<Item = &Signature> + Send + Sync
+    {
+        self.self_signatures.iter_bad(self.backsig_signer.as_ref())
+            .chain(self.self_revocations.iter_bad(self.backsig_signer.as_ref()))
     }
 
     /// Returns the component's revocation status at time `t`.
@@ -691,7 +662,6 @@ impl<C> ComponentBundle<C> {
     ///     even if there is a newer self-signature).
     ///
     /// selfsig must be the newest live self signature at time `t`.
-    #[allow(clippy::blocks_in_if_conditions)]
     pub(crate) fn _revocation_status<'a, T>(&'a self, policy: &dyn Policy, t: T,
                                             hard_revocations_are_final: bool,
                                             selfsig: Option<&Signature>)
@@ -908,11 +878,11 @@ impl<P: key::KeyParts, R: key::KeyRole> ComponentBundle<Key<P, R>> {
     /// #
     /// # fn main() -> openpgp::Result<()> {
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// // Display some information about the keys.
     /// for ka in cert.keys() {
-    ///     eprintln!(" - {:?}", ka.key());
+    ///     eprintln!(" - {:?}", ka.bundle().key());
     /// }
     /// # Ok(()) }
     /// ```
@@ -924,15 +894,14 @@ impl<P: key::KeyParts, R: key::KeyRole> ComponentBundle<Key<P, R>> {
     pub(crate) fn key_mut(&mut self) -> &mut Key<P, R> {
         self.component_mut()
     }
-}
 
-impl<P: key::KeyParts> ComponentBundle<Key<P, key::PrimaryRole>> {
-    /// Returns all of the bundles's bad signatures.
-    pub(crate) fn bad_signatures(&self)
-        -> impl Iterator<Item = &Signature> + Send + Sync
-    {
-        self.self_signatures.iter_bad(self.backsig_signer.as_ref())
-            .chain(self.self_revocations.iter_bad(self.backsig_signer.as_ref()))
+    pub(crate) fn set_role(&mut self, role: key::KeyRoleRT) {
+        self.key_mut().set_role(role);
+    }
+
+    /// Forwarder for the conversion macros.
+    pub(crate) fn has_secret(&self) -> bool {
+        self.key().has_secret()
     }
 }
 
@@ -989,12 +958,13 @@ impl<P: key::KeyParts> ComponentBundle<Key<P, key::SubordinateRole>> {
     /// let p = &StandardPolicy::new();
     ///
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// // Display the subkeys' revocation status.
     /// for ka in cert.keys().subkeys() {
     ///     eprintln!(" Revocation status of {}: {:?}",
-    ///               ka.fingerprint(), ka.revocation_status(p, None));
+    ///               ka.key().fingerprint(),
+    ///               ka.bundle().revocation_status(p, None));
     /// }
     /// # Ok(()) }
     /// ```
@@ -1006,14 +976,6 @@ impl<P: key::KeyParts> ComponentBundle<Key<P, key::SubordinateRole>> {
         self._revocation_status(policy, t, true,
                                 self.binding_signature(policy, t).ok())
     }
-
-    /// Returns all of the bundles's bad signatures.
-    pub(crate) fn bad_signatures(&self)
-        -> impl Iterator<Item = &Signature> + Send + Sync
-    {
-        self.self_signatures.iter_bad(self.backsig_signer.as_ref())
-            .chain(self.self_revocations.iter_bad(self.backsig_signer.as_ref()))
-    }
 }
 
 impl ComponentBundle<UserID> {
@@ -1032,11 +994,11 @@ impl ComponentBundle<UserID> {
     /// #
     /// # fn main() -> openpgp::Result<()> {
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// // Display some information about the User IDs.
     /// for ua in cert.userids() {
-    ///     eprintln!(" - {:?}", ua.userid());
+    ///     eprintln!(" - {:?}", ua.bundle().userid());
     /// }
     /// # Ok(()) }
     /// ```
@@ -1072,13 +1034,13 @@ impl ComponentBundle<UserID> {
     /// let p = &StandardPolicy::new();
     ///
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// // Display the User IDs' revocation status.
     /// for ua in cert.userids() {
     ///     eprintln!(" Revocation status of {}: {:?}",
     ///               String::from_utf8_lossy(ua.userid().value()),
-    ///               ua.revocation_status(p, None));
+    ///               ua.bundle().revocation_status(p, None));
     /// }
     /// # Ok(()) }
     /// ```
@@ -1089,14 +1051,6 @@ impl ComponentBundle<UserID> {
         let t = t.into();
         self._revocation_status(policy, t, false, self.binding_signature(policy, t).ok())
     }
-
-    /// Returns all of the bundles's bad signatures.
-    pub(crate) fn bad_signatures(&self)
-        -> impl Iterator<Item = &Signature> + Send + Sync
-    {
-        self.self_signatures.iter_bad(self.backsig_signer.as_ref())
-            .chain(self.self_revocations.iter_bad(self.backsig_signer.as_ref()))
-    }
 }
 
 impl ComponentBundle<UserAttribute> {
@@ -1115,11 +1069,11 @@ impl ComponentBundle<UserAttribute> {
     /// #
     /// # fn main() -> openpgp::Result<()> {
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// // Display some information about the User Attributes
     /// for ua in cert.user_attributes() {
-    ///     eprintln!(" - {:?}", ua.user_attribute());
+    ///     eprintln!(" - {:?}", ua.bundle().user_attribute());
     /// }
     /// # Ok(()) }
     /// ```
@@ -1151,12 +1105,12 @@ impl ComponentBundle<UserAttribute> {
     /// let p = &StandardPolicy::new();
     ///
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// // Display the User Attributes' revocation status.
     /// for (i, ua) in cert.user_attributes().enumerate() {
     ///     eprintln!(" Revocation status of User Attribute #{}: {:?}",
-    ///               i, ua.revocation_status(p, None));
+    ///               i, ua.bundle().revocation_status(p, None));
     /// }
     /// # Ok(()) }
     /// ```
@@ -1168,14 +1122,6 @@ impl ComponentBundle<UserAttribute> {
         self._revocation_status(policy, t, false,
                                 self.binding_signature(policy, t).ok())
     }
-
-    /// Returns all of the bundles's bad signatures.
-    pub(crate) fn bad_signatures(&self)
-        -> impl Iterator<Item = &Signature> + Send + Sync
-    {
-        self.self_signatures.iter_bad(self.backsig_signer.as_ref())
-            .chain(self.self_revocations.iter_bad(self.backsig_signer.as_ref()))
-    }
 }
 
 impl ComponentBundle<Unknown> {
@@ -1194,7 +1140,7 @@ impl ComponentBundle<Unknown> {
     /// #
     /// # fn main() -> openpgp::Result<()> {
     /// # let (cert, _) =
-    /// #     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #     CertBuilder::general_purpose(Some("alice@example.org"))
     /// #     .generate()?;
     /// // Display some information about the User Attributes
     /// for u in cert.unknowns() {
@@ -1205,14 +1151,6 @@ impl ComponentBundle<Unknown> {
     pub fn unknown(&self) -> &Unknown {
         self.component()
     }
-
-    /// Returns all of the bundles's bad signatures.
-    pub(crate) fn bad_signatures(&self)
-        -> impl Iterator<Item = &Signature> + Send + Sync
-    {
-        self.self_signatures.iter_bad(self.backsig_signer.as_ref())
-            .chain(self.self_revocations.iter_bad(self.backsig_signer.as_ref()))
-    }
 }
 
 /// A collection of `ComponentBundles`.
