@@ -1,5 +1,4 @@
 use std::convert::TryFrom;
-use std::ops::Deref;
 use std::time;
 
 use crate::{
@@ -36,7 +35,7 @@ use crate::cert::prelude::*;
 /// and the key used to sign the revocation certificate comes from
 /// `R`.
 ///
-/// [designated revoker]: https://tools.ietf.org/html/rfc4880#section-5.2.3.15
+/// [designated revoker]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.23
 ///
 /// # Examples
 ///
@@ -72,7 +71,7 @@ use crate::cert::prelude::*;
 ///     .build(&mut signer, &cert, None)?;
 ///
 /// // Merge it into the certificate.
-/// let cert = cert.insert_packets(sig.clone())?;
+/// let cert = cert.insert_packets(sig.clone())?.0;
 ///
 /// // Now it's revoked.
 /// assert_eq!(RevocationStatus::Revoked(vec![&sig]),
@@ -84,7 +83,6 @@ pub struct CertRevocationBuilder {
 }
 assert_send_and_sync!(CertRevocationBuilder);
 
-#[allow(clippy::new_without_default)]
 impl CertRevocationBuilder {
     /// Returns a new `CertRevocationBuilder`.
     ///
@@ -296,7 +294,7 @@ impl CertRevocationBuilder {
     /// # assert_eq!(sig.typ(), SignatureType::KeyRevocation);
     /// #
     /// # // Merge it into the certificate.
-    /// # let cert = cert.insert_packets(sig.clone())?;
+    /// # let cert = cert.insert_packets(sig.clone())?.0;
     /// #
     /// # // Now it's revoked.
     /// # assert_eq!(RevocationStatus::Revoked(vec![&sig]),
@@ -313,14 +311,6 @@ impl CertRevocationBuilder {
     }
 }
 
-impl Deref for CertRevocationBuilder {
-    type Target = signature::SignatureBuilder;
-
-    fn deref(&self) -> &Self::Target {
-        &self.builder
-    }
-}
-
 impl TryFrom<signature::SignatureBuilder> for CertRevocationBuilder {
     type Error = anyhow::Error;
 
@@ -358,7 +348,7 @@ impl TryFrom<signature::SignatureBuilder> for CertRevocationBuilder {
 /// of `A`, even if that subkey is not bound to `A`.  Semantically,
 /// such a revocation certificate is currently meaningless.
 ///
-/// [designated revoker]: https://tools.ietf.org/html/rfc4880#section-5.2.3.15
+/// [designated revoker]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.23
 ///
 /// # Examples
 ///
@@ -392,7 +382,7 @@ impl TryFrom<signature::SignatureBuilder> for CertRevocationBuilder {
 ///     .build(&mut signer, &cert, subkey.key(), None)?;
 ///
 /// // Merge it into the certificate.
-/// let cert = cert.insert_packets(sig.clone())?;
+/// let cert = cert.insert_packets(sig.clone())?.0;
 ///
 /// // Now it's revoked.
 /// let subkey = cert.keys().subkeys().nth(0).unwrap();
@@ -413,7 +403,6 @@ pub struct SubkeyRevocationBuilder {
 }
 assert_send_and_sync!(SubkeyRevocationBuilder);
 
-#[allow(clippy::new_without_default)]
 impl SubkeyRevocationBuilder {
     /// Returns a new `SubkeyRevocationBuilder`.
     ///
@@ -615,7 +604,7 @@ impl SubkeyRevocationBuilder {
     /// # assert_eq!(sig.typ(), SignatureType::SubkeyRevocation);
     /// #
     /// # // Merge it into the certificate.
-    /// # let cert = cert.insert_packets(sig.clone())?;
+    /// # let cert = cert.insert_packets(sig.clone())?.0;
     /// #
     /// # // Now it's revoked.
     /// # assert_eq!(RevocationStatus::Revoked(vec![&sig]),
@@ -636,14 +625,6 @@ impl SubkeyRevocationBuilder {
     }
 }
 
-impl Deref for SubkeyRevocationBuilder {
-    type Target = signature::SignatureBuilder;
-
-    fn deref(&self) -> &Self::Target {
-        &self.builder
-    }
-}
-
 impl TryFrom<signature::SignatureBuilder> for SubkeyRevocationBuilder {
     type Error = anyhow::Error;
 
@@ -683,7 +664,7 @@ impl TryFrom<signature::SignatureBuilder> for SubkeyRevocationBuilder {
 /// meaningless.
 ///
 /// [User ID]: crate::packet::UserID
-/// [designated revoker]: https://tools.ietf.org/html/rfc4880#section-5.2.3.15
+/// [designated revoker]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.23
 ///
 /// # Examples
 ///
@@ -717,7 +698,7 @@ impl TryFrom<signature::SignatureBuilder> for SubkeyRevocationBuilder {
 ///     .build(&mut signer, &cert, ua.userid(), None)?;
 ///
 /// // Merge it into the certificate.
-/// let cert = cert.insert_packets(sig.clone())?;
+/// let cert = cert.insert_packets(sig.clone())?.0;
 ///
 /// // Now it's revoked.
 /// let ua = cert.userids().nth(0).unwrap();
@@ -738,7 +719,6 @@ pub struct UserIDRevocationBuilder {
 }
 assert_send_and_sync!(UserIDRevocationBuilder);
 
-#[allow(clippy::new_without_default)]
 impl UserIDRevocationBuilder {
     /// Returns a new `UserIDRevocationBuilder`.
     ///
@@ -947,7 +927,7 @@ impl UserIDRevocationBuilder {
     /// # assert_eq!(sig.typ(), SignatureType::CertificationRevocation);
     /// #
     /// # // Merge it into the certificate.
-    /// # let cert = cert.insert_packets(sig.clone())?;
+    /// # let cert = cert.insert_packets(sig.clone())?.0;
     /// #
     /// # // Now it's revoked.
     /// # assert_eq!(RevocationStatus::Revoked(vec![&sig]),
@@ -967,14 +947,6 @@ impl UserIDRevocationBuilder {
     }
 }
 
-impl Deref for UserIDRevocationBuilder {
-    type Target = signature::SignatureBuilder;
-
-    fn deref(&self) -> &Self::Target {
-        &self.builder
-    }
-}
-
 impl TryFrom<signature::SignatureBuilder> for UserIDRevocationBuilder {
     type Error = anyhow::Error;
 
@@ -1015,7 +987,7 @@ impl TryFrom<signature::SignatureBuilder> for UserIDRevocationBuilder {
 /// meaningless.
 ///
 /// [User Attribute]: crate::packet::user_attribute
-/// [designated revoker]: https://tools.ietf.org/html/rfc4880#section-5.2.3.15
+/// [designated revoker]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.23
 ///
 /// # Examples
 ///
@@ -1056,7 +1028,7 @@ impl TryFrom<signature::SignatureBuilder> for UserIDRevocationBuilder {
 ///     .build(&mut signer, &cert, ua.user_attribute(), None)?;
 ///
 /// // Merge it into the certificate.
-/// let cert = cert.insert_packets(sig.clone())?;
+/// let cert = cert.insert_packets(sig.clone())?.0;
 ///
 /// // Now it's revoked.
 /// let ua = cert.user_attributes().nth(0).unwrap();
@@ -1077,7 +1049,6 @@ pub struct UserAttributeRevocationBuilder {
 }
 assert_send_and_sync!(UserAttributeRevocationBuilder);
 
-#[allow(clippy::new_without_default)]
 impl UserAttributeRevocationBuilder {
     /// Returns a new `UserAttributeRevocationBuilder`.
     ///
@@ -1292,7 +1263,7 @@ impl UserAttributeRevocationBuilder {
     /// # assert_eq!(sig.typ(), SignatureType::CertificationRevocation);
     /// #
     /// # // Merge it into the certificate.
-    /// # let cert = cert.insert_packets(sig.clone())?;
+    /// # let cert = cert.insert_packets(sig.clone())?.0;
     /// #
     /// # // Now it's revoked.
     /// # assert_eq!(RevocationStatus::Revoked(vec![&sig]),
@@ -1312,14 +1283,6 @@ impl UserAttributeRevocationBuilder {
     }
 }
 
-impl Deref for UserAttributeRevocationBuilder {
-    type Target = signature::SignatureBuilder;
-
-    fn deref(&self) -> &Self::Target {
-        &self.builder
-    }
-}
-
 impl TryFrom<signature::SignatureBuilder> for UserAttributeRevocationBuilder {
     type Error = anyhow::Error;
 
@@ -1428,10 +1391,10 @@ mod tests {
         // Create and sign a revocation certificate.
         let mut signer = cert.primary_key().key().clone()
             .parts_into_secret()?.into_keypair()?;
-        let user_id = cert.userids().next().unwrap();
+        let user_id = cert.userids().next().unwrap().userid();
         let builder = SignatureBuilder::new(SignatureType::CertificationRevocation);
         let revocation_builder: UserIDRevocationBuilder = builder.try_into()?;
-        let sig = revocation_builder.build(&mut signer, &cert, &user_id, None)?;
+        let sig = revocation_builder.build(&mut signer, &cert, user_id, None)?;
         assert_eq!(sig.typ(), SignatureType::CertificationRevocation);
         Ok(())
     }
@@ -1471,10 +1434,11 @@ mod tests {
         // Create and sign a revocation certificate.
         let mut signer = cert.primary_key().key().clone()
             .parts_into_secret()?.into_keypair()?;
-        let user_attribute = cert.user_attributes().next().unwrap();
+        let user_attribute =
+            cert.user_attributes().next().unwrap().user_attribute();
         let builder = SignatureBuilder::new(SignatureType::CertificationRevocation);
         let revocation_builder: UserAttributeRevocationBuilder = builder.try_into()?;
-        let sig = revocation_builder.build(&mut signer, &cert, &user_attribute, None)?;
+        let sig = revocation_builder.build(&mut signer, &cert, user_attribute, None)?;
         assert_eq!(sig.typ(), SignatureType::CertificationRevocation);
         Ok(())
     }
