@@ -17,6 +17,7 @@ use crate::Result;
 use crate::crypto::SessionKey;
 use crate::seal;
 use crate::parse::Cookie;
+use crate::crypto::backend::{Backend, interface::Kdf};
 
 /// Minimum AEAD chunk size.
 ///
@@ -57,6 +58,7 @@ pub trait Aead : seal::Sealed {
     fn encrypt_seal(&mut self, dst: &mut [u8], src: &[u8]) -> Result<()>;
 
     /// Length of the digest in bytes.
+    #[allow(dead_code)] // Used in debug assertions.
     fn digest_size(&self) -> usize;
 
     /// Decrypt one chunk `src` to `dst` and verify that the digest is
@@ -77,10 +79,11 @@ impl AEADAlgorithm {
     pub fn digest_size(&self) -> Result<usize> {
         use self::AEADAlgorithm::*;
         match self {
-            // According to RFC4880bis, Section 5.16.1.
+            // See https://www.rfc-editor.org/rfc/rfc9580.html#name-eax-mode
             EAX => Ok(16),
-            // According to RFC4880bis, Section 5.16.2.
+            // See https://www.rfc-editor.org/rfc/rfc9580.html#name-ocb-mode
             OCB => Ok(16),
+            // See https://www.rfc-editor.org/rfc/rfc9580.html#name-gcm-mode
             GCM => Ok(16),
             _ => Err(Error::UnsupportedAEADAlgorithm(*self).into()),
         }
@@ -90,25 +93,15 @@ impl AEADAlgorithm {
     pub fn nonce_size(&self) -> Result<usize> {
         use self::AEADAlgorithm::*;
         match self {
-            // According to RFC4880bis, Section 5.16.1.
+            // See https://www.rfc-editor.org/rfc/rfc9580.html#name-eax-mode
             EAX => Ok(16),
-            // According to RFC4880bis, Section 5.16.2, the IV is "at
-            // least 15 octets long".  GnuPG hardcodes 15 in
-            // openpgp_aead_algo_info.
+            // See https://www.rfc-editor.org/rfc/rfc9580.html#name-ocb-mode
             OCB => Ok(15),
+            // See https://www.rfc-editor.org/rfc/rfc9580.html#name-gcm-mode
             GCM => Ok(12),
             _ => Err(Error::UnsupportedAEADAlgorithm(*self).into()),
         }
     }
-
-    /// Returns the initialization vector size of the AEAD algorithm.
-    ///
-    /// This function is deprecated. Please use
-    /// [`AEADAlgorithm::nonce_size`].
-    #[deprecated = "Use nonce_size"]
-    pub fn iv_size(&self) -> Result<usize> {
-        self.nonce_size()
-    }
 }
 
 /// Schedules nonce and additional authenticated data for use with
@@ -135,67 +128,64 @@ pub trait Schedule: Send + Sync {
         F: FnMut(&[u8], &[u8]) -> R;
 }
 
-const AED1AD_PREFIX_LEN: usize = 5;
-pub(crate) struct AEDv1Schedule {
-    iv: Box<[u8]>,
-    ad: [u8; AED1AD_PREFIX_LEN],
+const SEIP2AD_PREFIX_LEN: usize = 5;
+pub(crate) struct SEIPv2Schedule {
+    nonce: Box<[u8]>,
+    ad: [u8; SEIP2AD_PREFIX_LEN],
+    nonce_len: usize,
 }
 
-impl AEDv1Schedule {
-    pub(crate) fn new(sym_algo: SymmetricAlgorithm,
+impl SEIPv2Schedule {
+    pub(crate) fn new(session_key: &SessionKey,
+                      sym_algo: SymmetricAlgorithm,
                       aead: AEADAlgorithm,
                       chunk_size: usize,
-                      iv: &[u8]) -> Result<Self>
+                      salt: &[u8]) -> Result<(SessionKey, Self)>
     {
         if !(MIN_CHUNK_SIZE..=MAX_CHUNK_SIZE).contains(&chunk_size) {
             return Err(Error::InvalidArgument(
                 format!("Invalid AEAD chunk size: {}", chunk_size)).into());
         }
 
-        Ok(Self {
-            iv: iv.into(),
-            ad: [
-                // Prefix.
-                0xd4, // Packet tag.
-                1,    // Version.
-                sym_algo.into(),
-                aead.into(),
-                chunk_size.trailing_zeros() as u8 - 6,
-            ],
-        })
+        // Derive the message key and initialization vector.
+        let key_size = sym_algo.key_size()?;
+        // The NONCE size is NONCE_LEN - 8 bytes taken from the KDF.
+        let nonce_size = aead.nonce_size()? - 8;
+        let mut key_nonce: SessionKey =
+            vec![0; key_size + nonce_size].into();
+        let ad = [
+            0xd2, // Tag.
+            2,    // Version.
+            sym_algo.into(),
+            aead.into(),
+            chunk_size.trailing_zeros() as u8 - 6,
+        ];
+        Backend::hkdf_sha256(session_key, Some(salt), &ad, &mut key_nonce)?;
+        let key = Vec::from(&key_nonce[..key_size]).into();
+        let nonce = Vec::from(&key_nonce[key_size..]).into();
+
+        Ok((key, Self {
+            nonce,
+            ad,
+            nonce_len: aead.nonce_size()?,
+        }))
     }
 }
 
-impl Schedule for AEDv1Schedule {
+impl Schedule for SEIPv2Schedule {
     fn next_chunk<F, R>(&self, index: u64, mut fun: F) -> R
     where
         F: FnMut(&[u8], &[u8]) -> R,
     {
-        // Prepare the associated data.
-        let mut ad = [0u8; AED1AD_PREFIX_LEN + 8];
-        ad[..AED1AD_PREFIX_LEN].copy_from_slice(&self.ad);
-        write_be_u64(&mut ad[AED1AD_PREFIX_LEN..],
-                     index);
-
-        // The nonce is computed by treating the starting
-        // initialization vector as a 16-octet, big-endian value and
-        // exclusive-oring the low eight octets of it with the chunk
-        // index.
-        let mut nonce_store = [0u8; MAX_NONCE_LEN];
-        let nonce_len = self.iv.len();
-        let nonce = &mut nonce_store[..nonce_len];
-        nonce.copy_from_slice(&self.iv);
+        // The nonce is the NONCE (NONCE_LEN - 8 bytes taken from the
+        // KDF) concatenated with the chunk index.
         let index_be: [u8; 8] = index.to_be_bytes();
-        for (i, o) in &mut nonce[nonce_len.saturating_sub(8)..].iter_mut()
-            .enumerate()
-        {
-            // The lower eight octets of the associated data
-            // are the big endian representation of the chunk
-            // index.
-            *o ^= index_be[i];
-        }
+        let mut nonce_store = [0u8; MAX_NONCE_LEN];
+        let nonce = &mut nonce_store[..self.nonce_len];
+        nonce[..self.nonce.len()].copy_from_slice(&self.nonce);
+        nonce[self.nonce.len()..].copy_from_slice(&index_be);
 
-        fun(nonce, &ad)
+        fun(nonce, &self.ad)
     }
 
     fn final_chunk<F, R>(&self, index: u64, length: u64, mut fun: F) -> R
@@ -203,30 +193,17 @@ impl Schedule for AEDv1Schedule {
         F: FnMut(&[u8], &[u8]) -> R,
     {
         // Prepare the associated data.
-        let mut ad = [0u8; AED1AD_PREFIX_LEN + 8 + 8];
-        ad[..AED1AD_PREFIX_LEN].copy_from_slice(&self.ad);
-        write_be_u64(&mut ad[AED1AD_PREFIX_LEN..AED1AD_PREFIX_LEN + 8],
-                     index);
-        write_be_u64(&mut ad[AED1AD_PREFIX_LEN + 8..],
-                     length);
+        let mut ad = [0u8; SEIP2AD_PREFIX_LEN + 8];
+        ad[..SEIP2AD_PREFIX_LEN].copy_from_slice(&self.ad);
+        write_be_u64(&mut ad[SEIP2AD_PREFIX_LEN..], length);
 
-        // The nonce is computed by treating the starting
-        // initialization vector as a 16-octet, big-endian value and
-        // exclusive-oring the low eight octets of it with the chunk
-        // index.
-        let mut nonce_store = [0u8; MAX_NONCE_LEN];
-        let nonce_len = self.iv.len();
-        let nonce = &mut nonce_store[..nonce_len];
-        nonce.copy_from_slice(&self.iv);
+        // The nonce is the NONCE (NONCE_LEN - 8 bytes taken from the
+        // KDF) concatenated with the chunk index.
         let index_be: [u8; 8] = index.to_be_bytes();
-        for (i, o) in &mut nonce[nonce_len.saturating_sub(8)..].iter_mut()
-            .enumerate()
-        {
-            // The lower eight octets of the associated data
-            // are the big endian representation of the chunk
-            // index.
-            *o ^= index_be[i];
-        }
+        let mut nonce_store = [0u8; MAX_NONCE_LEN];
+        let nonce = &mut nonce_store[..self.nonce_len];
+        nonce[..self.nonce.len()].copy_from_slice(&self.nonce);
+        nonce[self.nonce.len()..].copy_from_slice(&index_be);
 
         fun(nonce, &ad)
     }
@@ -806,17 +783,19 @@ mod tests {
             for aead in [
                 AEADAlgorithm::EAX,
                 AEADAlgorithm::OCB,
+                AEADAlgorithm::GCM,
             ].iter().filter(|algo| algo.is_supported() && algo.supports_symmetric_algo(sym_algo)) {
                 let chunk_size = 64;
                 let mut key = vec![0; sym_algo.key_size().unwrap()];
-                crate::crypto::random(&mut key);
+                crate::crypto::random(&mut key).unwrap();
                 let key: SessionKey = key.into();
                 let mut iv = vec![0; aead.nonce_size().unwrap()];
-                crate::crypto::random(&mut iv);
+                crate::crypto::random(&mut iv).unwrap();
 
                 let mut ciphertext = Vec::new();
                 {
-                    let schedule = AEDv1Schedule::new(
+                    let (message_key, schedule) = SEIPv2Schedule::new(
+                        &key,
                         *sym_algo,
                         *aead,
                         chunk_size,
@@ -825,7 +804,7 @@ mod tests {
                                                        *aead,
                                                        chunk_size,
                                                        schedule,
-                                                       key.clone(),
+                                                       message_key,
                                                        &mut ciphertext)
                         .unwrap();
 
@@ -834,7 +813,8 @@ mod tests {
 
                 let mut plaintext = Vec::new();
                 {
-                    let schedule = AEDv1Schedule::new(
+                    let (message_key, schedule) = SEIPv2Schedule::new(
+                        &key,
                         *sym_algo,
                         *aead,
                         chunk_size,
@@ -843,7 +823,7 @@ mod tests {
                                                        *aead,
                                                        chunk_size,
                                                        schedule,
-                                                       key,
+                                                       message_key,
                                                        Cursor::new(&ciphertext))
                         .unwrap();
 
