@@ -8,6 +8,7 @@ pub mod aead;
 pub mod asymmetric;
 pub mod ecdh;
 pub mod hash;
+pub mod kdf;
 pub mod symmetric;
 
 pub struct Backend(());
@@ -53,14 +54,27 @@ impl AEADAlgorithm {
                 match algo {
                     SymmetricAlgorithm::AES128 |
                     SymmetricAlgorithm::AES192 |
-                    SymmetricAlgorithm::AES256 |
-                    SymmetricAlgorithm::Twofish |
-                    SymmetricAlgorithm::Camellia128 |
-                    SymmetricAlgorithm::Camellia192 |
-                    SymmetricAlgorithm::Camellia256 => true,
+                    SymmetricAlgorithm::AES256 => true,
                     _ => false,
                 },
-            _ => false
+
+            AEADAlgorithm::OCB =>
+                match algo {
+                    SymmetricAlgorithm::AES128 |
+                    SymmetricAlgorithm::AES192 |
+                    SymmetricAlgorithm::AES256 => true,
+                    _ => false,
+                },
+
+            AEADAlgorithm::GCM =>
+                match algo {
+                    SymmetricAlgorithm::AES128 |
+                    SymmetricAlgorithm::AES192 |
+                    SymmetricAlgorithm::AES256 => true,
+                    _ => false,
+                },
+
+            _ => false,
         }
     }
 }
