@@ -4,7 +4,7 @@ use nettle::{ecc, ecdh, random::Yarrow};
 
 use crate::{Error, Result};
 use crate::crypto::SessionKey;
-use crate::crypto::ecdh::{encrypt_wrap, decrypt_unwrap2};
+use crate::crypto::ecdh::{encrypt_wrap, decrypt_unwrap};
 use crate::crypto::mem::Protected;
 use crate::crypto::mpi::{MPI, PublicKey, SecretKeyMaterial, Ciphertext};
 use crate::packet::{key, Key};
@@ -84,7 +84,7 @@ pub fn encrypt<R>(recipient: &Key<key::PublicParts, R>,
             }
 
             // Not implemented in Nettle
-            Curve::BrainpoolP256 | Curve::BrainpoolP512 =>
+            Curve::BrainpoolP256 | Curve::BrainpoolP384 | Curve::BrainpoolP512 =>
                 Err(Error::UnsupportedEllipticCurve(curve.clone()).into()),
 
             // N/A
@@ -163,7 +163,7 @@ pub fn decrypt<R>(recipient: &Key<key::PublicParts, R>,
                 }
 
                 // Not implemented in Nettle
-                Curve::BrainpoolP256 | Curve::BrainpoolP512 =>
+                Curve::BrainpoolP256 | Curve::BrainpoolP384 | Curve::BrainpoolP512 =>
                     return
                     Err(Error::UnsupportedEllipticCurve(curve.clone()).into()),
 
@@ -173,8 +173,8 @@ pub fn decrypt<R>(recipient: &Key<key::PublicParts, R>,
                     Err(Error::UnsupportedEllipticCurve(curve.clone()).into()),
             };
 
-            decrypt_unwrap2(recipient.role_as_unspecified(), &S, ciphertext,
-                            plaintext_len)
+            decrypt_unwrap(recipient.role_as_unspecified(), &S, ciphertext,
+                           plaintext_len)
         }
 
         _ =>
