@@ -3,8 +3,8 @@
 use std::cmp;
 use std::cmp::Ordering;
 
-use cipher::{BlockCipher, BlockEncrypt, BlockSizeUser, KeyInit, Unsigned};
-use cipher::consts::{U12, U16};
+use cipher::{BlockCipher, BlockDecrypt, BlockEncrypt, BlockSizeUser, KeyInit, Unsigned};
+use cipher::consts::{U12, U15, U16};
 use eax::online::{Eax, Encrypt, Decrypt};
 use cipher::generic_array::GenericArray;
 
@@ -139,6 +139,58 @@ where
     Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockEncrypt,
 {}
 
+struct Ocb<Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockDecrypt + BlockEncrypt> {
+    cipher: ocb3::Ocb3<Cipher, U15>,
+    nonce: GenericArray<u8, U15>,
+    aad: Vec<u8>,
+}
+
+impl<Cipher> Aead for Ocb<Cipher>
+where
+    Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockDecrypt + BlockEncrypt,
+{
+    fn digest_size(&self) -> usize {
+        TagLen::USIZE
+    }
+
+    fn encrypt_seal(&mut self, dst: &mut [u8], src: &[u8]) -> Result<()> {
+        debug_assert_eq!(dst.len(), src.len() + self.digest_size());
+        use ocb3::AeadInPlace;
+
+        let len = cmp::min(dst.len(), src.len());
+        dst[..len].copy_from_slice(&src[..len]);
+        let tag =
+            self.cipher.encrypt_in_place_detached(&self.nonce, &self.aad,
+                                                  &mut dst[..len])?;
+        debug_assert_eq!(dst[len..].len(), tag.len());
+        let tag_len = cmp::min(dst[len..].len(), tag.len());
+        dst[len..len + tag_len].copy_from_slice(&tag[..tag_len]);
+        Ok(())
+    }
+
+    fn decrypt_verify(&mut self, dst: &mut [u8], src: &[u8]) -> Result<()> {
+        debug_assert_eq!(dst.len() + self.digest_size(), src.len());
+        use ocb3::AeadInPlace;
+
+        // Split src into ciphertext and digest.
+        let len = src.len().saturating_sub(self.digest_size());
+        let digest = &src[len..];
+        let src = &src[..len];
+
+        debug_assert_eq!(dst.len(), src.len());
+        let len = core::cmp::min(dst.len(), src.len());
+        dst[..len].copy_from_slice(&src[..len]);
+        self.cipher.decrypt_in_place_detached(&self.nonce, &self.aad, dst,
+                                              digest.try_into()?)?;
+        Ok(())
+    }
+}
+
+impl<'a, Cipher> seal::Sealed for Ocb<Cipher>
+where
+    Cipher: BlockCipher + BlockSizeUser<BlockSize = U16> + BlockDecrypt + BlockEncrypt,
+{}
+
 impl AEADAlgorithm {
     pub(crate) fn context(
         &self,
@@ -148,6 +200,7 @@ impl AEADAlgorithm {
         nonce: &[u8],
         op: CipherOp,
     ) -> Result<Box<dyn Aead>> {
+        #[allow(deprecated)]
         match self {
             AEADAlgorithm::EAX => match sym_algo {
                 SymmetricAlgorithm::AES128 => match op {
@@ -269,8 +322,56 @@ impl AEADAlgorithm {
                     Err(Error::UnsupportedSymmetricAlgorithm(sym_algo).into()),
             },
 
-            AEADAlgorithm::OCB =>
-                Err(Error::UnsupportedAEADAlgorithm(*self).into()),
+            AEADAlgorithm::OCB => {
+                use ocb3::{Ocb3, Nonce};
+                match sym_algo {
+                    SymmetricAlgorithm::AES128 => {
+                        let nonce = Nonce::try_from_slice(nonce)?;
+                        let cipher =
+                            Ocb3::<aes::Aes128, U15>::new_from_slice(key)?;
+                        Ok(Box::new(Ocb { cipher, nonce: *nonce, aad: aad.to_vec() }))
+                    },
+                    SymmetricAlgorithm::AES192 => {
+                        let nonce = Nonce::try_from_slice(nonce)?;
+                        let cipher =
+                            Ocb3::<aes::Aes192, U15>::new_from_slice(key)?;
+                        Ok(Box::new(Ocb { cipher, nonce: *nonce, aad: aad.to_vec() }))
+                    },
+                    SymmetricAlgorithm::AES256 => {
+                        let nonce = Nonce::try_from_slice(nonce)?;
+                        let cipher =
+                            Ocb3::<aes::Aes256, U15>::new_from_slice(key)?;
+                        Ok(Box::new(Ocb { cipher, nonce: *nonce, aad: aad.to_vec() }))
+                    },
+                    SymmetricAlgorithm::Camellia128 => {
+                        let nonce = Nonce::try_from_slice(nonce)?;
+                        let cipher =
+                            Ocb3::<camellia::Camellia128, U15>::new_from_slice(key)?;
+                        Ok(Box::new(Ocb { cipher, nonce: *nonce, aad: aad.to_vec() }))
+                    },
+                    SymmetricAlgorithm::Camellia192 => {
+                        let nonce = Nonce::try_from_slice(nonce)?;
+                        let cipher =
+                            Ocb3::<camellia::Camellia192, U15>::new_from_slice(key)?;
+                        Ok(Box::new(Ocb { cipher, nonce: *nonce, aad: aad.to_vec() }))
+                    },
+                    SymmetricAlgorithm::Camellia256 => {
+                        let nonce = Nonce::try_from_slice(nonce)?;
+                        let cipher =
+                            Ocb3::<camellia::Camellia256, U15>::new_from_slice(key)?;
+                        Ok(Box::new(Ocb { cipher, nonce: *nonce, aad: aad.to_vec() }))
+                    },
+                    | SymmetricAlgorithm::IDEA
+                    | SymmetricAlgorithm::TripleDES
+                    | SymmetricAlgorithm::CAST5
+                    | SymmetricAlgorithm::Blowfish
+                    | SymmetricAlgorithm::Twofish
+                    | SymmetricAlgorithm::Private(_)
+                    | SymmetricAlgorithm::Unknown(_)
+                    | SymmetricAlgorithm::Unencrypted =>
+                        Err(Error::UnsupportedSymmetricAlgorithm(sym_algo).into()),
+                }
+            },
 
             AEADAlgorithm::GCM => {
                 use aes_gcm::{AesGcm, Nonce};
