@@ -4,17 +4,16 @@
 //! [ciphertexts], and [signatures] are scalar numbers of arbitrary
 //! precision.  OpenPGP specifies that these are stored encoded as
 //! big-endian integers with leading zeros stripped (See [Section 3.2
-//! of RFC 4880]).  Multiprecision integers in OpenPGP are extended by
-//! [RFC 6637] to store curves and coordinates used in elliptic curve
-//! cryptography (ECC).
+//! of RFC 9580]).  Multiprecision integers in OpenPGP are extended by
+//! [Section 3.2.1 of RFC 9580] to store curves and coordinates used
+//! in elliptic curve cryptography (ECC).
 //!
 //!   [public keys]: PublicKey
 //!   [secret keys]: SecretKeyMaterial
 //!   [ciphertexts]: Ciphertext
 //!   [signatures]: Signature
-//!   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
-//!   [RFC 6637]: https://tools.ietf.org/html/rfc6637
-
+//!   [Section 3.2 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-3.2
+//!   [Section 3.2.1 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-3.2.1
 use std::fmt;
 use std::cmp::Ordering;
 use std::io::Write;
@@ -88,7 +87,7 @@ impl MPI {
 
     /// Creates new MPI encoding an uncompressed EC point.
     ///
-    /// Encodes the given point on a elliptic curve (see [Section 6 of
+    /// Encodes the given point on an elliptic curve (see [Section 6 of
     /// RFC 6637] for details).  This is used to encode public keys
     /// and ciphertexts for the NIST curves (`NistP256`, `NistP384`,
     /// and `NistP521`).
@@ -114,7 +113,7 @@ impl MPI {
     /// Creates new MPI encoding a compressed EC point using native
     /// encoding.
     ///
-    /// Encodes the given point on a elliptic curve (see [Section 13.2
+    /// Encodes the given point on an elliptic curve (see [Section 13.2
     /// of RFC4880bis] for details).  This is used to encode public
     /// keys and ciphertexts for the Bernstein curves (currently
     /// `X25519`).
@@ -216,14 +215,11 @@ impl MPI {
                 Ok((&value[1..], &[]))
             },
 
-            Unknown(_) if ! curve.is_brainpoolp384() =>
-                Err(Error::UnsupportedEllipticCurve(curve.clone()).into()),
-
             NistP256
                 | NistP384
                 | NistP521
                 | BrainpoolP256
-                | Unknown(_)
+                | BrainpoolP384
                 | BrainpoolP512
                 =>
             {
@@ -251,6 +247,9 @@ impl MPI {
                 Ok((&value[1..1 + coordinate_length],
                     &value[1 + coordinate_length..]))
             },
+
+            Unknown(_) =>
+                Err(Error::UnsupportedEllipticCurve(curve.clone()).into()),
         }
     }
 
@@ -282,11 +281,12 @@ impl fmt::Debug for MPI {
 }
 
 impl Hash for MPI {
-    fn hash(&self, hash: &mut dyn hash::Digest) {
+    fn hash(&self, hash: &mut hash::Context) -> Result<()> {
         let len = self.bits() as u16;
 
         hash.update(&len.to_be_bytes());
         hash.update(&self.value);
+        Ok(())
     }
 }
 
@@ -380,16 +380,6 @@ impl From<Protected> for ProtectedMPI {
     }
 }
 
-// XXX: In 2.0, get rid of this conversion.  If the value has been
-// parsed into an MPI, it may have already leaked.
-impl From<MPI> for ProtectedMPI {
-    fn from(m: MPI) -> Self {
-        ProtectedMPI {
-            value: m.value.into(),
-        }
-    }
-}
-
 impl PartialOrd for ProtectedMPI {
     fn partial_cmp(&self, other: &ProtectedMPI) -> Option<Ordering> {
         Some(self.cmp(other))
@@ -416,10 +406,23 @@ impl std::hash::Hash for ProtectedMPI {
     }
 }
 
+#[cfg(test)]
+impl Arbitrary for ProtectedMPI {
+    fn arbitrary(g: &mut Gen) -> Self {
+        loop {
+            let buf = <Vec<u8>>::arbitrary(g);
+
+            if ! buf.is_empty() && buf[0] != 0 {
+                break ProtectedMPI::from(buf);
+            }
+        }
+    }
+}
+
 impl ProtectedMPI {
     /// Creates new MPI encoding an uncompressed EC point.
     ///
-    /// Encodes the given point on a elliptic curve (see [Section 6 of
+    /// Encodes the given point on an elliptic curve (see [Section 6 of
     /// RFC 6637] for details).  This is used to encode public keys
     /// and ciphertexts for the NIST curves (`NistP256`, `NistP384`,
     /// and `NistP521`).
@@ -432,7 +435,7 @@ impl ProtectedMPI {
     /// Creates new MPI encoding a compressed EC point using native
     /// encoding.
     ///
-    /// Encodes the given point on a elliptic curve (see [Section 13.2
+    /// Encodes the given point on an elliptic curve (see [Section 13.2
     /// of RFC4880bis] for details).  This is used to encode public
     /// keys and ciphertexts for the Bernstein curves (currently
     /// `X25519`).
@@ -520,9 +523,6 @@ impl fmt::Debug for ProtectedMPI {
 /// the occasional elliptic curve) in [`Key`] packets.
 ///
 ///   [`Key`]: crate::packet::Key
-///
-/// Note: This enum cannot be exhaustively matched to allow future
-/// extensions.
 #[non_exhaustive]
 #[derive(Clone, PartialEq, Eq, Hash, Debug, PartialOrd, Ord)]
 pub enum PublicKey {
@@ -584,6 +584,30 @@ pub enum PublicKey {
         sym: SymmetricAlgorithm,
     },
 
+    /// X25519 public key.
+    X25519 {
+        /// The public key, an opaque string.
+        u: [u8; 32],
+    },
+
+    /// X448 public key.
+    X448 {
+        /// The public key, an opaque string.
+        u: Box<[u8; 56]>,
+    },
+
+    /// Ed25519 public key.
+    Ed25519 {
+        /// The public key, an opaque string.
+        a: [u8; 32],
+    },
+
+    /// Ed448 public key.
+    Ed448 {
+        /// The public key, an opaque string.
+        a: Box<[u8; 57]>,
+    },
+
     /// Unknown number of MPIs for an unknown algorithm.
     Unknown {
         /// The successfully parsed MPIs.
@@ -611,9 +635,13 @@ impl PublicKey {
             RSA { ref n,.. } => Some(n.bits()),
             DSA { ref p,.. } => Some(p.bits()),
             ElGamal { ref p,.. } => Some(p.bits()),
-            EdDSA { ref curve,.. } => curve.bits(),
-            ECDSA { ref curve,.. } => curve.bits(),
-            ECDH { ref curve,.. } => curve.bits(),
+            EdDSA { ref curve,.. } => curve.bits().ok(),
+            ECDSA { ref curve,.. } => curve.bits().ok(),
+            ECDH { ref curve,.. } => curve.bits().ok(),
+            X25519 { .. } => Some(256),
+            X448 { .. } => Some(448),
+            Ed25519 { .. } => Some(256),
+            Ed448 { .. } => Some(456),
             Unknown { .. } => None,
         }
     }
@@ -622,6 +650,7 @@ impl PublicKey {
     /// key.
     pub fn algo(&self) -> Option<PublicKeyAlgorithm> {
         use self::PublicKey::*;
+        #[allow(deprecated)]
         match self {
             RSA { .. } => Some(PublicKeyAlgorithm::RSAEncryptSign),
             DSA { .. } => Some(PublicKeyAlgorithm::DSA),
@@ -629,15 +658,18 @@ impl PublicKey {
             EdDSA { .. } => Some(PublicKeyAlgorithm::EdDSA),
             ECDSA { .. } => Some(PublicKeyAlgorithm::ECDSA),
             ECDH { .. } => Some(PublicKeyAlgorithm::ECDH),
+            X25519 { .. } => Some(PublicKeyAlgorithm::X25519),
+            X448 { .. } => Some(PublicKeyAlgorithm::X448),
+            Ed25519 { .. } => Some(PublicKeyAlgorithm::Ed25519),
+            Ed448 { .. } => Some(PublicKeyAlgorithm::Ed448),
             Unknown { .. } => None,
         }
     }
 }
 
 impl Hash for PublicKey {
-    fn hash(&self, mut hash: &mut dyn hash::Digest) {
+    fn hash(&self, mut hash: &mut hash::Context) -> Result<()> {
         self.serialize(&mut hash as &mut dyn Write)
-            .expect("hashing does not fail")
     }
 }
 
@@ -647,7 +679,7 @@ impl Arbitrary for PublicKey {
         use self::PublicKey::*;
         use crate::arbitrary_helper::gen_arbitrary_from_range;
 
-        match gen_arbitrary_from_range(0..6, g) {
+        match gen_arbitrary_from_range(0..10, g) {
             0 => RSA {
                 e: MPI::arbitrary(g),
                 n: MPI::arbitrary(g),
@@ -683,11 +715,30 @@ impl Arbitrary for PublicKey {
                 sym: SymmetricAlgorithm::arbitrary(g),
             },
 
+            6 => X25519 { u: arbitrary(g) },
+            7 => X448 { u: Box::new(arbitrarize(g, [0; 56])) },
+            8 => Ed25519 { a: arbitrary(g) },
+            9 => Ed448 { a: Box::new(arbitrarize(g, [0; 57])) },
+
             _ => unreachable!(),
         }
     }
 }
 
+#[cfg(test)]
+pub(crate) fn arbitrarize<T: AsMut<[u8]>>(g: &mut Gen, mut a: T) -> T
+{
+    a.as_mut().iter_mut().for_each(|p| *p = Arbitrary::arbitrary(g));
+    a
+}
+
+#[cfg(test)]
+pub(crate) fn arbitrary<T: Default + AsMut<[u8]>>(g: &mut Gen) -> T
+{
+    arbitrarize(g, Default::default())
+}
+
+
 /// A secret key.
 ///
 /// Provides a typed and structured way of storing multiple MPIs in
@@ -695,9 +746,6 @@ impl Arbitrary for PublicKey {
 /// them using [`ProtectedMPI`].
 ///
 ///   [`Key`]: crate::packet::Key
-///
-/// Note: This enum cannot be exhaustively matched to allow future
-/// extensions.
 // Deriving Hash here is okay: PartialEq is manually implemented to
 // ensure that secrets are compared in constant-time.
 #[non_exhaustive]
@@ -746,6 +794,30 @@ pub enum SecretKeyMaterial {
         scalar: ProtectedMPI,
     },
 
+    /// X25519 secret key.
+    X25519 {
+        /// The secret key, an opaque string.
+        x: Protected,
+    },
+
+    /// X448 secret key.
+    X448 {
+        /// The secret key, an opaque string.
+        x: Protected,
+    },
+
+    /// Ed25519 secret key.
+    Ed25519 {
+        /// The secret key, an opaque string.
+        x: Protected,
+    },
+
+    /// Ed448 secret key.
+    Ed448 {
+        /// The secret key, an opaque string.
+        x: Protected,
+    },
+
     /// Unknown number of MPIs for an unknown algorithm.
     Unknown {
         /// The successfully parsed MPIs.
@@ -772,6 +844,14 @@ impl fmt::Debug for SecretKeyMaterial {
                     write!(f, "ECDSA {{ scalar: {:?} }}", scalar),
                 SecretKeyMaterial::ECDH{ ref scalar } =>
                     write!(f, "ECDH {{ scalar: {:?} }}", scalar),
+                SecretKeyMaterial::X25519 { x } =>
+                    write!(f, "X25519 {{ x: {:?} }}", x),
+                SecretKeyMaterial::X448 { x } =>
+                    write!(f, "X448 {{ x: {:?} }}", x),
+                SecretKeyMaterial::Ed25519 { x } =>
+                    write!(f, "Ed25519 {{ x: {:?} }}", x),
+                SecretKeyMaterial::Ed448 { x } =>
+                    write!(f, "Ed448 {{ x: {:?} }}", x),
                 SecretKeyMaterial::Unknown{ ref mpis, ref rest } =>
                     write!(f, "Unknown {{ mips: {:?}, rest: {:?} }}", mpis, rest),
             }
@@ -789,6 +869,14 @@ impl fmt::Debug for SecretKeyMaterial {
                     f.write_str("ECDSA { <Redacted> }"),
                 SecretKeyMaterial::ECDH{ .. } =>
                     f.write_str("ECDH { <Redacted> }"),
+                SecretKeyMaterial::X25519 { .. } =>
+                    f.write_str("X25519 { <Redacted> }"),
+                SecretKeyMaterial::X448 { .. } =>
+                    f.write_str("X448 { <Redacted> }"),
+                SecretKeyMaterial::Ed25519 { .. } =>
+                    f.write_str("Ed25519 { <Redacted> }"),
+                SecretKeyMaterial::Ed448 { .. } =>
+                    f.write_str("Ed448 { <Redacted> }"),
                 SecretKeyMaterial::Unknown{ .. } =>
                     f.write_str("Unknown { <Redacted> }"),
             }
@@ -814,7 +902,11 @@ impl Ord for SecretKeyMaterial {
                 SecretKeyMaterial::EdDSA{ .. } => 3,
                 SecretKeyMaterial::ECDSA{ .. } => 4,
                 SecretKeyMaterial::ECDH{ .. } => 5,
-                SecretKeyMaterial::Unknown{ .. } => 6,
+                SecretKeyMaterial::X25519 { .. } => 6,
+                SecretKeyMaterial::X448 { .. } => 7,
+                SecretKeyMaterial::Ed25519 { .. } => 8,
+                SecretKeyMaterial::Ed448 { .. } => 9,
+                SecretKeyMaterial::Unknown { .. } => 10,
             }
         }
 
@@ -851,6 +943,15 @@ impl Ord for SecretKeyMaterial {
             ,&SecretKeyMaterial::ECDH{ scalar: ref scalar2 }) => {
                 scalar1.cmp(scalar2)
             }
+            (SecretKeyMaterial::X25519 { x: x0 },
+             SecretKeyMaterial::X25519 { x: x1 }) => x0.cmp(x1),
+            (SecretKeyMaterial::X448 { x: x0 },
+             SecretKeyMaterial::X448 { x: x1 }) => x0.cmp(x1),
+            (SecretKeyMaterial::Ed25519 { x: x0 },
+             SecretKeyMaterial::Ed25519 { x: x1 }) => x0.cmp(x1),
+            (SecretKeyMaterial::Ed448 { x: x0 },
+             SecretKeyMaterial::Ed448 { x: x1 }) => x0.cmp(x1),
+
             (&SecretKeyMaterial::Unknown{ mpis: ref mpis1, rest: ref rest1 }
             ,&SecretKeyMaterial::Unknown{ mpis: ref mpis2, rest: ref rest2 }) => {
                 let o1 = secure_cmp(rest1, rest2);
@@ -888,6 +989,7 @@ impl SecretKeyMaterial {
     /// key.
     pub fn algo(&self) -> Option<PublicKeyAlgorithm> {
         use self::SecretKeyMaterial::*;
+        #[allow(deprecated)]
         match self {
             RSA { .. } => Some(PublicKeyAlgorithm::RSAEncryptSign),
             DSA { .. } => Some(PublicKeyAlgorithm::DSA),
@@ -895,15 +997,18 @@ impl SecretKeyMaterial {
             EdDSA { .. } => Some(PublicKeyAlgorithm::EdDSA),
             ECDSA { .. } => Some(PublicKeyAlgorithm::ECDSA),
             ECDH { .. } => Some(PublicKeyAlgorithm::ECDH),
+            X25519 { .. } => Some(PublicKeyAlgorithm::X25519),
+            X448 { .. } => Some(PublicKeyAlgorithm::X448),
+            Ed25519 { .. } => Some(PublicKeyAlgorithm::Ed25519),
+            Ed448 { .. } => Some(PublicKeyAlgorithm::Ed448),
             Unknown { .. } => None,
         }
     }
 }
 
 impl Hash for SecretKeyMaterial {
-    fn hash(&self, mut hash: &mut dyn hash::Digest) {
+    fn hash(&self, mut hash: &mut hash::Context) -> Result<()> {
         self.serialize(&mut hash as &mut dyn Write)
-            .expect("hashing does not fail")
     }
 }
 
@@ -914,30 +1019,43 @@ impl SecretKeyMaterial {
         #[allow(deprecated)]
         match pk {
             RSAEncryptSign | RSASign | RSAEncrypt => Ok(SecretKeyMaterial::RSA {
-                d: MPI::arbitrary(g).into(),
-                p: MPI::arbitrary(g).into(),
-                q: MPI::arbitrary(g).into(),
-                u: MPI::arbitrary(g).into(),
+                d: ProtectedMPI::arbitrary(g),
+                p: ProtectedMPI::arbitrary(g),
+                q: ProtectedMPI::arbitrary(g),
+                u: ProtectedMPI::arbitrary(g),
             }),
 
             DSA => Ok(SecretKeyMaterial::DSA {
-                x: MPI::arbitrary(g).into(),
+                x: ProtectedMPI::arbitrary(g),
             }),
 
             ElGamalEncryptSign | ElGamalEncrypt => Ok(SecretKeyMaterial::ElGamal {
-                x: MPI::arbitrary(g).into(),
+                x: ProtectedMPI::arbitrary(g),
             }),
 
             EdDSA => Ok(SecretKeyMaterial::EdDSA {
-                scalar: MPI::arbitrary(g).into(),
+                scalar: ProtectedMPI::arbitrary(g),
             }),
 
             ECDSA => Ok(SecretKeyMaterial::ECDSA {
-                scalar: MPI::arbitrary(g).into(),
+                scalar: ProtectedMPI::arbitrary(g),
             }),
 
             ECDH => Ok(SecretKeyMaterial::ECDH {
-                scalar: MPI::arbitrary(g).into(),
+                scalar: ProtectedMPI::arbitrary(g),
+            }),
+
+            X25519 => Ok(SecretKeyMaterial::X25519 {
+                x: arbitrarize(g, vec![0; 32]).into(),
+            }),
+            X448 => Ok(SecretKeyMaterial::X448 {
+                x: arbitrarize(g, vec![0; 56]).into(),
+            }),
+            Ed25519 => Ok(SecretKeyMaterial::Ed25519 {
+                x: arbitrarize(g, vec![0; 32]).into(),
+            }),
+            Ed448 => Ok(SecretKeyMaterial::Ed448 {
+                x: arbitrarize(g, vec![0; 57]).into(),
             }),
 
             Private(_) | Unknown(_) =>
@@ -948,7 +1066,8 @@ impl SecretKeyMaterial {
 #[cfg(test)]
 impl Arbitrary for SecretKeyMaterial {
     fn arbitrary(g: &mut Gen) -> Self {
-        let pk = *g.choose(&crate::types::PUBLIC_KEY_ALGORITHM_VARIANTS)
+        let pk = *g.choose(
+            &crate::crypto::types::public_key_algorithm::PUBLIC_KEY_ALGORITHM_VARIANTS)
             .expect("not empty");
         Self::arbitrary_for(g, pk).expect("only known variants")
     }
@@ -957,9 +1076,9 @@ impl Arbitrary for SecretKeyMaterial {
 /// Checksum method for secret key material.
 ///
 /// Secret key material may be protected by a checksum.  See [Section
-/// 5.5.3 of RFC 4880] for details.
+/// 5.5.3 of RFC 9580] for details.
 ///
-///   [Section 5.5.3 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.5.3
+///   [Section 5.5.3 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.5.3
 #[derive(PartialEq, Eq, Hash, Clone, Copy, Debug)]
 pub enum SecretKeyChecksum {
     /// SHA1 over the decrypted secret key.
@@ -992,9 +1111,6 @@ impl SecretKeyChecksum {
 /// [`PKESK`] packets.
 ///
 ///   [`PKESK`]: crate::packet::PKESK
-///
-/// Note: This enum cannot be exhaustively matched to allow future
-/// extensions.
 #[non_exhaustive]
 #[derive(Clone, PartialEq, Eq, Hash, Debug, PartialOrd, Ord)]
 pub enum Ciphertext {
@@ -1020,6 +1136,22 @@ pub enum Ciphertext {
         key: Box<[u8]>,
     },
 
+    /// X25519 ciphertext.
+    X25519 {
+        /// Ephermeral key.
+        e: Box<[u8; 32]>,
+        /// Symmetrically encrypted session key.
+        key: Box<[u8]>,
+    },
+
+    /// X448 ciphertext.
+    X448 {
+        /// Ephermeral key.
+        e: Box<[u8; 56]>,
+        /// Symmetrically encrypted session key.
+        key: Box<[u8]>,
+    },
+
     /// Unknown number of MPIs for an unknown algorithm.
     Unknown {
         /// The successfully parsed MPIs.
@@ -1039,19 +1171,21 @@ impl Ciphertext {
         // Fields are mostly MPIs that consist of two octets length
         // plus the big endian value itself. All other field types are
         // commented.
+        #[allow(deprecated)]
         match self {
             RSA { .. } => Some(PublicKeyAlgorithm::RSAEncryptSign),
             ElGamal { .. } => Some(PublicKeyAlgorithm::ElGamalEncrypt),
             ECDH { .. } => Some(PublicKeyAlgorithm::ECDH),
+            X25519 { .. } => Some(PublicKeyAlgorithm::X25519),
+            X448 { .. } => Some(PublicKeyAlgorithm::X448),
             Unknown { .. } => None,
         }
     }
 }
 
 impl Hash for Ciphertext {
-    fn hash(&self, mut hash: &mut dyn hash::Digest) {
+    fn hash(&self, mut hash: &mut hash::Context) -> Result<()> {
         self.serialize(&mut hash as &mut dyn Write)
-            .expect("hashing does not fail")
     }
 }
 
@@ -1060,7 +1194,7 @@ impl Arbitrary for Ciphertext {
     fn arbitrary(g: &mut Gen) -> Self {
         use crate::arbitrary_helper::gen_arbitrary_from_range;
 
-        match gen_arbitrary_from_range(0..3, g) {
+        match gen_arbitrary_from_range(0..5, g) {
             0 => Ciphertext::RSA {
                 c: MPI::arbitrary(g),
             },
@@ -1078,6 +1212,24 @@ impl Arbitrary for Ciphertext {
                     k.into_boxed_slice()
                 },
             },
+
+            3 => Ciphertext::X25519 {
+                e: Box::new(arbitrary(g)),
+                key: {
+                    let mut k = <Vec<u8>>::arbitrary(g);
+                    k.truncate(255);
+                    k.into_boxed_slice()
+                },
+            },
+
+            4 => Ciphertext::X448 {
+                e: Box::new(arbitrarize(g, [0; 56])),
+                key: {
+                    let mut k = <Vec<u8>>::arbitrary(g);
+                    k.truncate(255);
+                    k.into_boxed_slice()
+                },
+            },
             _ => unreachable!(),
         }
     }
@@ -1089,9 +1241,6 @@ impl Arbitrary for Ciphertext {
 /// [`Signature`] packets.
 ///
 ///   [`Signature`]: crate::packet::Signature
-///
-/// Note: This enum cannot be exhaustively matched to allow future
-/// extensions.
 #[non_exhaustive]
 #[derive(Clone, PartialEq, Eq, Hash, Debug, PartialOrd, Ord)]
 pub enum Signature {
@@ -1133,6 +1282,18 @@ pub enum Signature {
         s: MPI,
     },
 
+    /// Ed25519 signature.
+    Ed25519 {
+        /// The signature.
+        s: Box<[u8; 64]>,
+    },
+
+    /// Ed448 signature.
+    Ed448 {
+        /// The signature.
+        s: Box<[u8; 114]>,
+    },
+
     /// Unknown number of MPIs for an unknown algorithm.
     Unknown {
         /// The successfully parsed MPIs.
@@ -1144,9 +1305,8 @@ pub enum Signature {
 assert_send_and_sync!(Signature);
 
 impl Hash for Signature {
-    fn hash(&self, mut hash: &mut dyn hash::Digest) {
+    fn hash(&self, mut hash: &mut hash::Context) -> Result<()> {
         self.serialize(&mut hash as &mut dyn Write)
-            .expect("hashing does not fail")
     }
 }
 
@@ -1155,7 +1315,7 @@ impl Arbitrary for Signature {
     fn arbitrary(g: &mut Gen) -> Self {
         use crate::arbitrary_helper::gen_arbitrary_from_range;
 
-        match gen_arbitrary_from_range(0..4, g) {
+        match gen_arbitrary_from_range(0..6, g) {
             0 => Signature::RSA  {
                 s: MPI::arbitrary(g),
             },
@@ -1175,6 +1335,14 @@ impl Arbitrary for Signature {
                 s: MPI::arbitrary(g),
             },
 
+            4 => Signature::Ed25519  {
+                s: Box::new(arbitrarize(g, [0; 64])),
+            },
+
+            5 => Signature::Ed448  {
+                s: Box::new(arbitrarize(g, [0; 114])),
+            },
+
             _ => unreachable!(),
         }
     }
@@ -1216,6 +1384,14 @@ mod tests {
                     PublicKey::parse(ECDSA, cur).unwrap(),
                 PublicKey::ECDH { .. } =>
                     PublicKey::parse(ECDH, cur).unwrap(),
+                PublicKey::X25519 { .. } =>
+                    PublicKey::parse(X25519, cur).unwrap(),
+                PublicKey::X448 { .. } =>
+                    PublicKey::parse(X448, cur).unwrap(),
+                PublicKey::Ed25519 { .. } =>
+                    PublicKey::parse(Ed25519, cur).unwrap(),
+                PublicKey::Ed448 { .. } =>
+                    PublicKey::parse(Ed448, cur).unwrap(),
 
                 PublicKey::Unknown { .. } => unreachable!(),
             };
@@ -1266,6 +1442,14 @@ mod tests {
                     SecretKeyMaterial::parse(ECDH, cur).unwrap(),
                 SecretKeyMaterial::ElGamal { .. } =>
                     SecretKeyMaterial::parse(ElGamalEncrypt, cur).unwrap(),
+                SecretKeyMaterial::X25519 { .. } =>
+                    SecretKeyMaterial::parse(X25519, cur).unwrap(),
+                SecretKeyMaterial::X448 { .. } =>
+                    SecretKeyMaterial::parse(X448, cur).unwrap(),
+                SecretKeyMaterial::Ed25519 { .. } =>
+                    SecretKeyMaterial::parse(Ed25519, cur).unwrap(),
+                SecretKeyMaterial::Ed448 { .. } =>
+                    SecretKeyMaterial::parse(Ed448, cur).unwrap(),
 
                 SecretKeyMaterial::Unknown { .. } => unreachable!(),
             };
@@ -1291,6 +1475,10 @@ mod tests {
                     Ciphertext::parse(ElGamalEncrypt, cur).unwrap(),
                 Ciphertext::ECDH { .. } =>
                     Ciphertext::parse(ECDH, cur).unwrap(),
+                Ciphertext::X25519 { .. } =>
+                    Ciphertext::parse(X25519, cur).unwrap(),
+                Ciphertext::X448 { .. } =>
+                    Ciphertext::parse(X448, cur).unwrap(),
 
                 Ciphertext::Unknown { .. } => unreachable!(),
             };
@@ -1320,6 +1508,10 @@ mod tests {
                     Signature::parse(EdDSA, cur).unwrap(),
                 Signature::ECDSA { .. } =>
                     Signature::parse(ECDSA, cur).unwrap(),
+                Signature::Ed25519 { .. } =>
+                    Signature::parse(Ed25519, cur).unwrap(),
+                Signature::Ed448 { .. } =>
+                    Signature::parse(Ed448, cur).unwrap(),
 
                 Signature::Unknown { .. } => unreachable!(),
             };
