@@ -15,21 +15,20 @@ use crate::{
 /// This is needed because signatures can reference their issuer
 /// either by `Fingerprint` or by `KeyID`.
 ///
-/// Currently, Sequoia supports *version 4* fingerprints and Key ID
-/// only.  *Version 3* fingerprints and Key ID were deprecated by [RFC
-/// 4880] in 2007.
+/// Currently, Sequoia supports *version 6* fingerprints and Key IDs,
+/// and *version 4* fingerprints and Key IDs.  *Version 3*
+/// fingerprints and Key IDs were deprecated by [RFC 4880] in 2007.
 ///
-/// A *v4* fingerprint is, essentially, a 20-byte SHA-1 hash over the
-/// key's public key packet.  A *v4* Key ID is defined as the
-/// fingerprint's lower 8 bytes.
-///
-/// For the exact definition, see [Section 12.2 of RFC 4880].
+/// Essentially, a fingerprint is a hash over the key's public key
+/// packet.  *Version 6* and *version 4* [`KeyID`]s are a truncated
+/// version of the key's fingerprint. For details, see [Section 5.5.4
+/// of RFC 9580].
 ///
 /// Both fingerprint and Key ID are used to identify a key, e.g., the
 /// issuer of a signature.
 ///
 ///   [RFC 4880]: https://tools.ietf.org/html/rfc4880
-///   [Section 12.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-12.2
+///   [Section 5.5.4 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.5.4
 ///
 /// # A Note on Equality
 ///
@@ -134,6 +133,16 @@ impl From<KeyHandle> for KeyID {
     }
 }
 
+impl From<Option<KeyHandle>> for KeyID {
+    fn from(i: Option<KeyHandle>) -> Self {
+        match i {
+            Some(KeyHandle::Fingerprint(i)) => i.into(),
+            Some(KeyHandle::KeyID(i)) => i,
+            None => KeyID::wildcard(),
+        }
+    }
+}
+
 impl From<&KeyHandle> for KeyID {
     fn from(i: &KeyHandle) -> Self {
         match i {
@@ -196,8 +205,8 @@ impl std::str::FromStr for KeyHandle {
 
     fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
         let bytes = &crate::fmt::hex::decode_pretty(s)?[..];
-        match Fingerprint::from_bytes(bytes) {
-            fpr @ Fingerprint::Invalid(_) => {
+        match Fingerprint::from_bytes_intern(None, bytes)? {
+            fpr @ Fingerprint::Unknown { .. } => {
                 match KeyID::from_bytes(bytes) {
                     // If it can't be parsed as either a Fingerprint or a
                     // KeyID, return Fingerprint::Invalid.
@@ -268,9 +277,31 @@ impl KeyHandle {
     /// assert_ne!(fpr1, fpr2);
     /// assert_eq!(KeyID::from(&fpr1), KeyID::from(&fpr2));
     ///
+    /// # let v6_fpr1 : KeyHandle
+    /// #     = "AACB3243630052D9AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
+    /// #       .parse::<Fingerprint>()?.into();
+    /// #
+    /// # let v6_fpr2 : KeyHandle
+    /// #     = "AACB3243630052D9BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB"
+    /// #       .parse::<Fingerprint>()?.into();
+    /// #
+    /// # let keyid : KeyHandle = "AACB 3243 6300 52D9".parse::<KeyID>()?
+    /// #     .into();
+    /// #
+    /// // fpr1 and fpr2 are different v4 fingerprints with the same KeyID.
+    /// assert!(! fpr1.eq(&fpr2));
     /// assert!(fpr1.aliases(&keyid));
     /// assert!(fpr2.aliases(&keyid));
     /// assert!(! fpr1.aliases(&fpr2));
+    ///
+    /// // v6_fpr1 and v6_fpr2 are different v6 fingerprints with the same KeyID.
+    /// assert!(! v6_fpr1.eq(&v6_fpr2));
+    /// assert!(v6_fpr1.aliases(&keyid));
+    /// assert!(v6_fpr2.aliases(&keyid));
+    /// assert!(! v6_fpr1.aliases(&v6_fpr2));
+    ///
+    /// // And of course, v4 and v6 don't alias.
+    /// assert!(! fpr1.aliases(&v6_fpr1));
     /// # Ok(()) }
     /// ```
     pub fn aliases<H>(&self, other: H) -> bool
@@ -317,7 +348,7 @@ impl KeyHandle {
     /// ```
     pub fn is_invalid(&self) -> bool {
         matches!(self,
-                 KeyHandle::Fingerprint(Fingerprint::Invalid(_))
+                 KeyHandle::Fingerprint(Fingerprint::Unknown { .. })
                  | KeyHandle::KeyID(KeyID::Invalid(_)))
     }
 
@@ -452,10 +483,13 @@ mod tests {
             8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20]));
         assert_eq!(format!("{:X}", handle), "0102030405060708090A0B0C0D0E0F1011121314");
 
-        let handle = KeyHandle::Fingerprint(Fingerprint::Invalid(Box::new([10, 2, 3, 4])));
+        let handle = KeyHandle::Fingerprint(Fingerprint::Unknown {
+            version: None,
+            bytes: Box::new([10, 2, 3, 4]),
+        });
         assert_eq!(format!("{:X}", handle), "0A020304");
 
-        let handle = KeyHandle::KeyID(KeyID::V4([10, 2, 3, 4, 5, 6, 7, 8]));
+        let handle = KeyHandle::KeyID(KeyID::Long([10, 2, 3, 4, 5, 6, 7, 8]));
         assert_eq!(format!("{:X}", handle), "0A02030405060708");
 
         let handle = KeyHandle::KeyID(KeyID::Invalid(Box::new([10, 2])));
@@ -468,10 +502,13 @@ mod tests {
             8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20]));
         assert_eq!(format!("{:x}", handle), "0102030405060708090a0b0c0d0e0f1011121314");
 
-        let handle = KeyHandle::Fingerprint(Fingerprint::Invalid(Box::new([10, 2, 3, 4])));
+        let handle = KeyHandle::Fingerprint(Fingerprint::Unknown {
+            version: None,
+            bytes: Box::new([10, 2, 3, 4]),
+        });
         assert_eq!(format!("{:x}", handle), "0a020304");
 
-        let handle = KeyHandle::KeyID(KeyID::V4([10, 2, 3, 4, 5, 6, 7, 8]));
+        let handle = KeyHandle::KeyID(KeyID::Long([10, 2, 3, 4, 5, 6, 7, 8]));
         assert_eq!(format!("{:x}", handle), "0a02030405060708");
 
         let handle = KeyHandle::KeyID(KeyID::Invalid(Box::new([10, 2])));
@@ -488,14 +525,14 @@ mod tests {
                     0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 0x01, 0x23, 0x45, 0x67]);
 
         let handle: KeyHandle = "89AB CDEF 0123 4567".parse()?;
-        assert_match!(&KeyHandle::KeyID(KeyID::V4(_)) = &handle);
+        assert_match!(&KeyHandle::KeyID(KeyID::Long(_)) = &handle);
         assert_eq!(handle.as_bytes(),
                    [0x89, 0xAB, 0xCD, 0xEF, 0x01, 0x23, 0x45, 0x67]);
 
         // Invalid handles are parsed as invalid Fingerprints, not
         // invalid KeyIDs.
         let handle: KeyHandle = "4567 89AB CDEF 0123 4567".parse()?;
-        assert_match!(&KeyHandle::Fingerprint(Fingerprint::Invalid(_)) = &handle);
+        assert_match!(&KeyHandle::Fingerprint(Fingerprint::Unknown { .. }) = &handle);
         assert_eq!(handle.as_bytes(),
                    [0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 0x01, 0x23, 0x45, 0x67]);
 
