@@ -3,7 +3,7 @@
 //! An OpenPGP message is a sequence of OpenPGP packets that
 //! corresponds to an optionally signed, optionally encrypted,
 //! optionally compressed literal data packet.  The exact format of an
-//! OpenPGP message is described in [Section 11.3 of RFC 4880].
+//! OpenPGP message is described in [Section 10.3 of RFC 9580].
 //!
 //! This module provides support for validating and working with
 //! OpenPGP messages.
@@ -19,15 +19,10 @@
 //! -----END PGP MESSAGE-----
 //! ```
 //!
-//! [Section 11.3 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-11.3
-
-// XXX: For Token.  Drop this once Token is private.
-#![allow(deprecated)]
+//! [Section 10.3 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-10.3
 
 use std::convert::TryFrom;
 use std::fmt;
-use std::io;
-use std::path::Path;
 
 use buffered_reader::BufferedReader;
 
@@ -40,22 +35,19 @@ use crate::packet::Tag;
 use crate::parse::{Cookie, Parse};
 
 mod lexer;
-lalrpop_util::lalrpop_mod!(#[allow(clippy::all, deprecated)] grammar, "/message/grammar.rs");
+lalrpop_util::lalrpop_mod!(#[allow(clippy::all)] grammar, "/message/grammar.rs");
 
 use self::lexer::{Lexer, LexicalError};
-pub use self::lexer::Token;
+pub(crate) use self::lexer::Token;
 
 use lalrpop_util::ParseError;
 
 use self::grammar::MessageParser;
 
 /// Errors that MessageValidator::check may return.
-///
-/// Note: This enum cannot be exhaustively matched to allow future
-/// extensions.
 #[non_exhaustive]
 #[derive(Debug, Clone)]
-pub enum MessageParserError {
+pub (crate) enum MessageParserError {
     /// A parser error.
     Parser(ParseError<usize, Token, LexicalError>),
     /// An OpenPGP error.
@@ -207,6 +199,7 @@ impl MessageValidator {
             Tag::SKESK => Token::SKESK,
             Tag::PKESK => Token::PKESK,
             Tag::SEIP if version == Some(1) => Token::SEIPv1,
+            Tag::SEIP if version == Some(2) => Token::SEIPv2,
             Tag::MDC => Token::MDC,
             Tag::AED => Token::AED,
             Tag::OnePassSig => Token::OPS,
@@ -216,6 +209,16 @@ impl MessageValidator {
                 // section 5.8 of RFC4880.
                 return;
             },
+            Tag::Padding => {
+                // "[Padding packets] MUST be ignored when received.",
+                // section 5.14 of RFC 9580.
+                return;
+            },
+            t if ! t.is_critical() => {
+                // "Unknown, non-critical packets MUST be ignored when
+                // received.", section 4.3 of RFC 9580.
+                return;
+            },
             _ => {
                 // Unknown token.
                 self.error = Some(MessageParserError::OpenPGP(
@@ -290,10 +293,10 @@ impl MessageValidator {
 ///
 /// An OpenPGP message is a structured sequence of OpenPGP packets.
 /// Basically, it's an optionally encrypted, optionally signed literal
-/// data packet.  The exact structure is defined in [Section 11.3 of RFC
-/// 4880].
+/// data packet.  The exact structure is defined in [Section 10.3 of RFC
+/// 9580].
 ///
-///   [Section 11.3 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-11.3
+///   [Section 10.3 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-10.3
 ///
 /// [ASCII Armored Messages] are wrapped in `-----BEGIN PGP MESSAGE-----` header
 /// and `-----END PGP MESSAGE-----` tail lines:
@@ -309,7 +312,7 @@ impl MessageValidator {
 /// -----END PGP MESSAGE-----
 /// ```
 ///
-/// [ASCII Armored Messages]: https://tools.ietf.org/html/rfc4880#section-6.6
+/// [ASCII Armored Messages]: https://www.rfc-editor.org/rfc/rfc9580.html#section-!
 ///
 /// # Examples
 ///
@@ -354,34 +357,7 @@ impl<'a> Parse<'a, Message> for Message {
     where
         R: BufferedReader<Cookie> + 'a,
     {
-        Self::try_from(PacketPile::from_buffered_reader(reader)?)
-    }
-
-    /// Reads a `Message` from the specified reader.
-    ///
-    /// See [`Message::try_from`] for more details.
-    ///
-    ///   [`Message::try_from`]: Message::try_from()
-    fn from_reader<R: 'a + io::Read + Send + Sync>(reader: R) -> Result<Self> {
-        Self::try_from(PacketPile::from_reader(reader)?)
-    }
-
-    /// Reads a `Message` from the specified file.
-    ///
-    /// See [`Message::try_from`] for more details.
-    ///
-    ///   [`Message::try_from`]: Message::try_from()
-    fn from_file<P: AsRef<Path>>(path: P) -> Result<Self> {
-        Self::try_from(PacketPile::from_file(path)?)
-    }
-
-    /// Reads a `Message` from `buf`.
-    ///
-    /// See [`Message::try_from`] for more details.
-    ///
-    ///   [`Message::try_from`]: Message::try_from()
-    fn from_bytes<D: AsRef<[u8]> + ?Sized + Send + Sync>(data: &'a D) -> Result<Self> {
-        Self::try_from(PacketPile::from_bytes(data)?)
+        Self::try_from(PacketPile::from_buffered_reader(reader.into_boxed())?)
     }
 }
 
@@ -433,6 +409,11 @@ impl Message {
         // No literal data packet found.
         None
     }
+
+    /// Returns a reference to the message's packets.
+    pub fn packets(&self) -> &PacketPile {
+        &self.pile
+    }
 }
 
 impl TryFrom<PacketPile> for Message {
@@ -443,13 +424,13 @@ impl TryFrom<PacketPile> for Message {
     /// Converting a `PacketPile` to a `Message` doesn't change the
     /// packets; it asserts that the packet sequence is an optionally
     /// encrypted, optionally signed, optionally compressed literal
-    /// data packet.  The exact grammar is defined in [Section 11.3 of
-    /// RFC 4880].
+    /// data packet.  The exact grammar is defined in [Section 10.3 of
+    /// RFC 9580].
     ///
     /// Caveats: this function assumes that any still encrypted parts
     /// or still compressed parts are valid messages.
     ///
-    ///   [Section 11.3 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-11.3
+    ///   [Section 10.3 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-10.3
     fn try_from(pile: PacketPile) -> Result<Self> {
         let mut v = MessageValidator::new();
         for (mut path, packet) in pile.descendants().paths() {
@@ -465,7 +446,7 @@ impl TryFrom<PacketPile> for Message {
             }
 
             match packet {
-                Packet::CompressedData(_) | Packet::SEIP(_) | Packet::AED(_) =>
+                Packet::CompressedData(_) | Packet::SEIP(_) =>
                 {
                     // If a container's content is not unpacked, then
                     // we treat the content as an opaque message.
@@ -510,19 +491,11 @@ impl From<Message> for PacketPile {
     }
 }
 
-impl ::std::ops::Deref for Message {
-    type Target = PacketPile;
-
-    fn deref(&self) -> &Self::Target {
-        &self.pile
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
 
-    use crate::types::DataFormat::Text;
+    use crate::types::DataFormat::Unicode;
     use crate::HashAlgorithm;
     use crate::types::CompressionAlgorithm;
     use crate::SymmetricAlgorithm;
@@ -588,32 +561,32 @@ mod tests {
             },
 
             TestVector {
-                s: &[AED, Literal, Pop],
+                s: &[SEIPv2, Literal, Pop],
                 result: true,
             },
             TestVector {
-                s: &[CompressedData, AED, Literal, Pop, Pop],
+                s: &[CompressedData, SEIPv2, Literal, Pop, Pop],
                 result: true,
             },
             TestVector {
-                s: &[CompressedData, AED, CompressedData, Literal,
+                s: &[CompressedData, SEIPv2, CompressedData, Literal,
                      Pop, Pop, Pop],
                 result: true,
             },
             TestVector {
-                s: &[AED, Pop],
+                s: &[SEIPv2, Pop],
                 result: false,
             },
             TestVector {
-                s: &[SKESK, AED, Literal, Pop],
+                s: &[SKESK, SEIPv2, Literal, Pop],
                 result: true,
             },
             TestVector {
-                s: &[PKESK, AED, Literal, Pop],
+                s: &[PKESK, SEIPv2, Literal, Pop],
                 result: true,
             },
             TestVector {
-                s: &[SKESK, SKESK, AED, Literal, Pop],
+                s: &[SKESK, SKESK, SEIPv2, Literal, Pop],
                 result: true,
             },
 
@@ -831,7 +804,7 @@ mod tests {
         // 0: Literal
         // => good.
         let mut packets = Vec::new();
-        let mut lit = Literal::new(Text);
+        let mut lit = Literal::new(Unicode);
         lit.set_body(b"data".to_vec());
         packets.push(lit.into());
 
@@ -841,7 +814,7 @@ mod tests {
 
     #[test]
     fn compressed_part() {
-        let mut lit = Literal::new(Text);
+        let mut lit = Literal::new(Unicode);
         lit.set_body(b"data".to_vec());
 
         // 0: CompressedData
@@ -903,16 +876,18 @@ mod tests {
 
     #[test]
     fn one_pass_sig_part() {
-        let mut lit = Literal::new(Text);
+        let mut lit = Literal::new(Unicode);
         lit.set_body(b"data".to_vec());
 
         let hash = crate::types::HashAlgorithm::SHA512;
         let key: key::SecretKey =
-            crate::packet::key::Key4::generate_ecc(true, crate::types::Curve::Ed25519)
+            crate::packet::key::Key6::generate_ecc(true, crate::types::Curve::Ed25519)
             .unwrap().into();
         let mut pair = key.clone().into_keypair().unwrap();
+        let ctx =
+            hash.context().unwrap().for_signature(pair.public().version());
         let sig = crate::packet::signature::SignatureBuilder::new(SignatureType::Binary)
-            .sign_hash(&mut pair, hash.context().unwrap()).unwrap();
+            .sign_hash(&mut pair, ctx).unwrap();
 
         // 0: OnePassSig
         // => bad.
@@ -1015,16 +990,18 @@ mod tests {
 
     #[test]
     fn signature_part() {
-        let mut lit = Literal::new(Text);
+        let mut lit = Literal::new(Unicode);
         lit.set_body(b"data".to_vec());
 
         let hash = crate::types::HashAlgorithm::SHA512;
         let key: key::SecretKey =
-            crate::packet::key::Key4::generate_ecc(true, crate::types::Curve::Ed25519)
+            crate::packet::key::Key6::generate_ecc(true, crate::types::Curve::Ed25519)
             .unwrap().into();
         let mut pair = key.clone().into_keypair().unwrap();
+        let ctx =
+            hash.context().unwrap().for_signature(pair.public().version());
         let sig = crate::packet::signature::SignatureBuilder::new(SignatureType::Binary)
-            .sign_hash(&mut pair, hash.context().unwrap()).unwrap();
+            .sign_hash(&mut pair, ctx).unwrap();
 
         // 0: Signature
         // => bad.
@@ -1065,14 +1042,15 @@ mod tests {
         // internal interfaces to progressively build up more
         // complicated messages.
 
-        let mut lit = Literal::new(Text);
+        let mut lit = Literal::new(Unicode);
         lit.set_body(b"data".to_vec());
 
         // 0: SK-ESK
         // => bad.
         let mut packets : Vec<Packet> = Vec::new();
         let cipher = SymmetricAlgorithm::AES256;
-        let sk = crate::crypto::SessionKey::new(cipher.key_size().unwrap());
+        let sk =
+            crate::crypto::SessionKey::new(cipher.key_size().unwrap()).unwrap();
         #[allow(deprecated)]
         packets.push(SKESK4::with_password(
             cipher,
@@ -1100,9 +1078,9 @@ mod tests {
         //  1: MDC
         // => good.
         let mut seip = SEIP1::new();
-        seip.children_mut().unwrap().push(
+        seip.container_mut().children_mut().unwrap().push(
             lit.clone().into());
-        seip.children_mut().unwrap().push(
+        seip.container_mut().children_mut().unwrap().push(
             MDC::from([0u8; 20]).into());
         packets[1] = seip.into();
 
@@ -1204,7 +1182,7 @@ mod tests {
         packets.insert(
             1,
             PKESK3::new(
-                "0000111122223333".parse().unwrap(),
+                Some("0000111122223333".parse().unwrap()),
                 PublicKeyAlgorithm::RSAEncrypt,
                 Ciphertext::RSA { c: MPI::new(&[]) }).unwrap().into());
 
