@@ -3,11 +3,11 @@ use std::fmt;
 use std::str;
 use std::hash::{Hash, Hasher};
 use std::cmp::Ordering;
+use std::sync::OnceLock;
 
 #[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
 
-use once_cell::sync::OnceCell;
 use regex::Regex;
 
 use crate::Result;
@@ -33,7 +33,7 @@ impl ConventionallyParsedUserID {
     pub fn new<S>(userid: S) -> Result<Self>
         where S: Into<String>
     {
-        Self::parse(userid.into())
+        Ok(Self::parse(userid.into())?)
     }
 
     /// Returns the User ID's name component, if any.
@@ -60,8 +60,10 @@ impl ConventionallyParsedUserID {
     }
 
     fn parse(userid: String) -> Result<Self> {
-        lazy_static::lazy_static!{
-            static ref USER_ID_PARSER: Regex = {
+        fn user_id_parser() -> &'static Regex {
+            use std::sync::OnceLock;
+            static USER_ID_PARSER: OnceLock<Regex> = OnceLock::new();
+            USER_ID_PARSER.get_or_init(|| {
                 // Whitespace.
                 let ws_bare = " ";
                 let ws = format!("[{}]", ws_bare);
@@ -216,12 +218,12 @@ impl ConventionallyParsedUserID {
                               bare_name);
 
                 Regex::new(&pgp_uid_convention).unwrap()
-            };
+            })
         }
 
         // The regex is anchored at the start and at the end so we
         // have either 0 or 1 matches.
-        if let Some(cap) = USER_ID_PARSER.captures_iter(&userid).next() {
+        if let Some(cap) = user_id_parser().captures_iter(&userid).next() {
             let to_range = |m: regex::Match| (m.start(), m.end());
 
             // We need to figure out which branch matched.  Match on a
@@ -300,7 +302,7 @@ impl ConventionallyParsedUserID {
 /// Holds a UserID packet.
 ///
 /// The standard imposes no structure on UserIDs, but suggests to
-/// follow [RFC 2822].  See [Section 5.11 of RFC 4880] for details.
+/// follow [RFC 2822].  See [Section 5.11 of RFC 9580] for details.
 /// In practice though, implementations do not follow [RFC 2822], or
 /// do not even help their users in producing well-formed User IDs.
 /// Experience has shown that parsing User IDs using [RFC 2822] does
@@ -308,7 +310,7 @@ impl ConventionallyParsedUserID {
 /// what we call *Conventional User IDs*.
 ///
 ///   [RFC 2822]: https://tools.ietf.org/html/rfc2822
-///   [Section 5.11 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.11
+///   [Section 5.11 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.11
 ///
 /// Using this definition, we provide methods to extract the [name],
 /// [comment], [email address], or [URI] from `UserID` packets.
@@ -465,18 +467,18 @@ pub struct UserID {
     pub(crate) common: packet::Common,
     /// The user id.
     ///
-    /// According to [RFC 4880], the text is by convention UTF-8 encoded
+    /// According to [Section 5.11 of RFC 9580], the text is by convention UTF-8 encoded
     /// and in "mail name-addr" form, i.e., "Name (Comment)
     /// <email@example.com>".
     ///
-    ///   [RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.11
+    ///   [Section 5.11 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.11
     ///
     /// Use `UserID::default()` to get a UserID with a default settings.
     value: Cow<'static, [u8]>,
 
-    hash_algo_security: OnceCell<HashAlgoSecurity>,
+    hash_algo_security: OnceLock<HashAlgoSecurity>,
 
-    parsed: OnceCell<ConventionallyParsedUserID>,
+    parsed: OnceLock<ConventionallyParsedUserID>,
 }
 assert_send_and_sync!(UserID);
 
@@ -497,9 +499,9 @@ impl UserID {
     pub const fn from_static_bytes(u: &'static [u8]) -> Self {
         UserID {
             common: packet::Common::new(),
-            hash_algo_security: OnceCell::new(),
+            hash_algo_security: OnceLock::new(),
             value: Cow::Borrowed(u),
-            parsed: OnceCell::new(),
+            parsed: OnceLock::new(),
         }
     }
 }
@@ -597,9 +599,9 @@ impl Clone for UserID {
             hash_algo_security: self.hash_algo_security.clone(),
             value: self.value.clone(),
             parsed: if let Some(p) = self.parsed.get() {
-                OnceCell::with_value(p.clone())
+                p.clone().into()
             } else {
-                OnceCell::new()
+                OnceLock::new()
             },
         }
     }
@@ -801,7 +803,7 @@ impl UserID {
     /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::packet::UserID;
     /// assert_eq!(UserID::from_address(
-    ///                "John Smith".into(),
+    ///                "John Smith",
     ///                None,
     ///                "boat@example.org")?.value(),
     ///            &b"John Smith <boat@example.org>"[..]);
@@ -813,10 +815,12 @@ impl UserID {
     ///            &b"John Smith (Who is Advok?) <boat@example.org>"[..]);
     /// # Ok(()) }
     /// ```
-    pub fn from_address<O, S>(name: O, comment: O, email: S)
+    pub fn from_address<'a, N, C, E>(name: N, comment: C, email: E)
         -> Result<Self>
-        where S: AsRef<str>,
-              O: Into<Option<S>>
+    where
+        N: Into<Option<&'a str>>,
+        C: Into<Option<&'a str>>,
+        E: AsRef<str>,
     {
         Self::assemble(name.into().as_ref().map(|s| s.as_ref()),
                        comment.into().as_ref().map(|s| s.as_ref()),
@@ -843,15 +847,17 @@ impl UserID {
     /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::packet::UserID;
     /// assert_eq!(UserID::from_unchecked_address(
-    ///                "NAS".into(),
+    ///                "NAS",
     ///                None, "ssh://host.example.org")?.value(),
     ///            &b"NAS <ssh://host.example.org>"[..]);
     /// # Ok(()) }
     /// ```
-    pub fn from_unchecked_address<O, S>(name: O, comment: O, address: S)
+    pub fn from_unchecked_address<'a, N, C, E>(name: N, comment: C, address: E)
         -> Result<Self>
-        where S: AsRef<str>,
-              O: Into<Option<S>>
+    where
+        N: Into<Option<&'a str>>,
+        C: Into<Option<&'a str>>,
+        E: AsRef<str>,
     {
         Self::assemble(name.into().as_ref().map(|s| s.as_ref()),
                        comment.into().as_ref().map(|s| s.as_ref()),
@@ -878,15 +884,14 @@ impl UserID {
     }
 
     fn do_parse(&self) -> Result<&ConventionallyParsedUserID> {
-        self.parsed.get_or_try_init(|| {
-            let s = str::from_utf8(&self.value)?;
+        if let Some(p) = self.parsed.get() {
+            return Ok(p);
+        }
 
-            ConventionallyParsedUserID::new(s).map_err(|err| {
-                // Return the error from the NameAddrOrOther parser.
-                err.context(format!(
-                    "Failed to parse User ID: {:?}", s))
-            })
-        })
+        let s = str::from_utf8(&self.value)?;
+        let p = ConventionallyParsedUserID::parse(s.to_string())?;
+        let _lost_race = self.parsed.set(p.clone());
+        Ok(self.parsed.get().expect("just set"))
     }
 
     /// Parses the User ID according to de facto conventions, and
@@ -895,76 +900,40 @@ impl UserID {
     /// See [conventional User ID] for more information.
     ///
     ///   [conventional User ID]: #conventional-user-ids
-    pub fn name2(&self) -> Result<Option<&str>> {
+    pub fn name(&self) -> Result<Option<&str>> {
         Ok(self.do_parse()?.name())
     }
 
-    /// Parses the User ID according to de facto conventions, and
-    /// returns the name component, if any.
-    ///
-    /// Like [`UserID::name2`], but heap-allocates.
-    #[deprecated(note = "Use UserID::name2")]
-    pub fn name(&self) -> Result<Option<String>> {
-        Ok(self.do_parse()?.name().map(|s| s.to_string()))
-    }
-
     /// Parses the User ID according to de facto conventions, and
     /// returns the comment field, if any.
     ///
     /// See [conventional User ID] for more information.
     ///
     ///   [conventional User ID]: #conventional-user-ids
-    pub fn comment2(&self) -> Result<Option<&str>> {
+    pub fn comment(&self) -> Result<Option<&str>> {
         Ok(self.do_parse()?.comment())
     }
 
-    /// Parses the User ID according to de facto conventions, and
-    /// returns the comment field, if any.
-    ///
-    /// Like [`UserID::comment2`], but heap-allocates.
-    #[deprecated(note = "Use UserID::comment2")]
-    pub fn comment(&self) -> Result<Option<String>> {
-        Ok(self.do_parse()?.comment().map(|s| s.to_string()))
-    }
-
     /// Parses the User ID according to de facto conventions, and
     /// returns the email address, if any.
     ///
     /// See [conventional User ID] for more information.
     ///
     ///   [conventional User ID]: #conventional-user-ids
-    pub fn email2(&self) -> Result<Option<&str>> {
+    pub fn email(&self) -> Result<Option<&str>> {
         Ok(self.do_parse()?.email())
     }
 
-    /// Parses the User ID according to de facto conventions, and
-    /// returns the email address, if any.
-    ///
-    /// Like [`UserID::email2`], but heap-allocates.
-    #[deprecated(note = "Use UserID::email2")]
-    pub fn email(&self) -> Result<Option<String>> {
-        Ok(self.do_parse()?.email().map(|s| s.to_string()))
-    }
-
     /// Parses the User ID according to de facto conventions, and
     /// returns the URI, if any.
     ///
     /// See [conventional User ID] for more information.
     ///
     ///   [conventional User ID]: #conventional-user-ids
-    pub fn uri2(&self) -> Result<Option<&str>> {
+    pub fn uri(&self) -> Result<Option<&str>> {
         Ok(self.do_parse()?.uri())
     }
 
-    /// Parses the User ID according to de facto conventions, and
-    /// returns the URI, if any.
-    ///
-    /// Like [`UserID::uri2`], but heap-allocates.
-    #[deprecated(note = "Use UserID::uri2")]
-    pub fn uri(&self) -> Result<Option<String>> {
-        Ok(self.do_parse()?.uri().map(|s| s.to_string()))
-    }
-
     /// Returns a normalized version of the UserID's email address.
     ///
     /// Normalized email addresses are primarily needed when email
@@ -984,7 +953,7 @@ impl UserID {
     ///   [empty locale]: https://www.w3.org/International/wiki/Case_folding
     ///   [Autocrypt]: https://autocrypt.org/level1.html#e-mail-address-canonicalization
     pub fn email_normalized(&self) -> Result<Option<String>> {
-        match self.email2()? {
+        match self.email()? {
             None => Ok(None),
             Some(address) => {
                 let mut iter = address.split('@');
@@ -1342,7 +1311,7 @@ mod tests {
           None, None, Some("name@example.org"), None);
     }
 
-    // Make sure we can't parse non conventional User IDs.
+    // Make sure we can't parse non-conventional User IDs.
     #[test]
     fn decompose_non_conventional() {
         // Empty string is not allowed.
@@ -1436,7 +1405,7 @@ mod tests {
                        .unwrap().value(),
                    b"<foo@bar.com>");
         assert!(UserID::from_address(None, None, "foo@@bar.com").is_err());
-        assert_eq!(UserID::from_address("Foo Q. Bar".into(), None, "foo@bar.com")
+        assert_eq!(UserID::from_address("Foo Q. Bar", None, "foo@bar.com")
                       .unwrap().value(),
                    b"Foo Q. Bar <foo@bar.com>");
     }
