@@ -1,6 +1,3 @@
-use std::io;
-use std::path::Path;
-
 use buffered_reader::BufferedReader;
 
 use crate::Result;
@@ -19,12 +16,13 @@ use crate::packet;
 ///
 /// When parsing OpenPGP data streams, the [`PacketParser`] will by
 /// default automatically detect and remove any ASCII armor encoding
-/// (see [Section 6 of RFC 4880]).  This automatism can be disabled
+/// (see [Section 6 of RFC 9580]).  This automatism can be disabled
 /// and fine-tuned using [`PacketParserBuilder::dearmor`].
 ///
-///   [Section 6 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-6
+///   [Section 6 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-6
 ///   [`PacketParserBuilder::dearmor`]: PacketParserBuilder::dearmor()
 #[derive(PartialEq)]
+#[non_exhaustive]
 pub enum Dearmor {
     /// Unconditionally treat the input as if it were an OpenPGP
     /// message encoded using ASCII armor.
@@ -109,31 +107,10 @@ impl<'a> Parse<'a, PacketParserBuilder<'a>> for PacketParserBuilder<'a> {
     {
         PacketParserBuilder::from_cookie_reader(reader.into_boxed())
     }
-
-    /// Creates a `PacketParserBuilder` for an OpenPGP message stored
-    /// in a `std::io::Read` object.
-    fn from_reader<R: io::Read + 'a + Send + Sync>(reader: R) -> Result<Self> {
-        PacketParserBuilder::from_cookie_reader(
-            Box::new(buffered_reader::Generic::with_cookie(
-                reader, None, Cookie::default())))
-    }
-
-    /// Creates a `PacketParserBuilder` for an OpenPGP message stored
-    /// in the file named `path`.
-    fn from_file<P: AsRef<Path>>(path: P) -> Result<Self> {
-        PacketParserBuilder::from_cookie_reader(
-            Box::new(buffered_reader::File::with_cookie(path, Cookie::default())?))
-    }
-
-    /// Creates a `PacketParserBuilder` for an OpenPGP message stored
-    /// in the specified buffer.
-    fn from_bytes<D: AsRef<[u8]> + ?Sized>(data: &'a D) -> Result<PacketParserBuilder<'a>> {
-        PacketParserBuilder::from_cookie_reader(
-            Box::new(buffered_reader::Memory::with_cookie(
-                data.as_ref(), Cookie::default())))
-    }
 }
 
+impl<'a> crate::seal::Sealed for PacketParserBuilder<'a> {}
+
 impl<'a> PacketParserBuilder<'a> {
     // Creates a `PacketParserBuilder` for an OpenPGP message stored
     // in a `BufferedReader` object.
