@@ -3,8 +3,8 @@
 //! OpenPGP defines a [regular expression language].  It is used with
 //! [trust signatures] to scope the trust that they extend.
 //!
-//!   [regular expression language]: https://tools.ietf.org/html/rfc4880#section-8
-//!   [trust signatures]: https://tools.ietf.org/html/rfc4880#section-5.2.3.13
+//!   [regular expression language]: https://www.rfc-editor.org/rfc/rfc9580.html#section-8
+//!   [trust signatures]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.21
 //!
 //! Compared with most regular expression languages, OpenPGP's is
 //! quite simple.  In particular, it only includes the following
@@ -54,12 +54,12 @@
 //! [Trust Signature] subpacket using, for instance, the
 //! [`SignatureBuilder::set_trust_signature`] method.
 //!
-//!   [type]: https://tools.ietf.org/html/rfc4880#section-5.2.1
+//!   [type]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.1
 //!   [GenericCertification]: crate::types::SignatureType::GenericCertification
 //!   [PersonaCertification]: crate::types::SignatureType::PersonaCertification
 //!   [CasualCertification]: crate::types::SignatureType::CasualCertification
 //!   [PositiveCertification]: crate::types::SignatureType::PositiveCertification
-//!   [Trust Signature]: https://tools.ietf.org/html/rfc4880#section-5.2.3.13
+//!   [Trust Signature]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.21
 //!   [`SignatureBuilder::set_trust_signature`]: crate::packet::signature::SignatureBuilder::set_trust_signature()
 //!
 //! To scope a trust signature, you add a [Regular Expression
@@ -70,7 +70,7 @@
 //! To extract any regular expressions, you can use
 //! [`SubpacketAreas::regular_expressions`].
 //!
-//!   [Regular Expression subpacket]: https://tools.ietf.org/html/rfc4880#section-5.2.3.14
+//!   [Regular Expression subpacket]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.22
 //!   [`SignatureBuilder::set_regular_expression`]: crate::packet::signature::SignatureBuilder::set_regular_expression()
 //!   [`SignatureBuilder::add_regular_expression`]: crate::packet::signature::SignatureBuilder::add_regular_expression()
 //!   [`SubpacketAreas::regular_expressions`]: crate::packet::signature::subpacket::SubpacketAreas::regular_expressions()
@@ -119,7 +119,7 @@
 //! let p = &StandardPolicy::new();
 //!
 //! let (ca, _)
-//!     = CertBuilder::general_purpose(None, Some("OpenPGP CA <openpgp-ca@example.com>"))
+//!     = CertBuilder::general_purpose(Some("OpenPGP CA <openpgp-ca@example.com>"))
 //!         .generate()?;
 //! let mut ca_signer = ca.primary_key().key().clone()
 //!     .parts_into_secret()?.into_keypair()?;
@@ -128,7 +128,7 @@
 //!
 //! // The CA certifies "Alice <alice@example.com>".
 //! let (alice, _)
-//!     = CertBuilder::general_purpose(None, Some("Alice <alice@example.com>"))
+//!     = CertBuilder::general_purpose(Some("Alice <alice@example.com>"))
 //!         .generate()?;
 //! let alice_userid = alice.with_policy(p, None)?
 //!     .userids().nth(0).expect("Added a User ID").userid();
@@ -137,8 +137,8 @@
 //!         &mut ca_signer,
 //!         alice.primary_key().component(),
 //!         alice_userid)?;
-//! let alice = alice.insert_packets(alice_certification.clone())?;
-//! # assert!(alice.clone().into_packets2().any(|p| {
+//! let alice = alice.insert_packets(alice_certification.clone())?.0;
+//! # assert!(alice.clone().into_packets().any(|p| {
 //! #   match p {
 //! #       Packet::Signature(sig) => sig == alice_certification,
 //! #       _ => false,
@@ -147,7 +147,7 @@
 //!
 //! // The CA certifies "Bob <bob@some.org>".
 //! let (bob, _)
-//!     = CertBuilder::general_purpose(None, Some("Bob <bob@some.org>"))
+//!     = CertBuilder::general_purpose(Some("Bob <bob@some.org>"))
 //!         .generate()?;
 //! let bob_userid = bob.with_policy(p, None)?
 //!     .userids().nth(0).expect("Added a User ID").userid();
@@ -156,8 +156,8 @@
 //!         &mut ca_signer,
 //!         bob.primary_key().component(),
 //!         bob_userid)?;
-//! let bob = bob.insert_packets(bob_certification.clone())?;
-//! # assert!(bob.clone().into_packets2().any(|p| {
+//! let bob = bob.insert_packets(bob_certification.clone())?.0;
+//! # assert!(bob.clone().into_packets().any(|p| {
 //! #   match p {
 //! #       Packet::Signature(sig) => sig == bob_certification,
 //! #       _ => false,
@@ -167,7 +167,7 @@
 //!
 //! // Carol tsigns the CA's certificate.
 //! let (carol, _)
-//!     = CertBuilder::general_purpose(None, Some("Carol <carol@another.net>"))
+//!     = CertBuilder::general_purpose(Some("Carol <carol@another.net>"))
 //!         .generate()?;
 //! let mut carol_signer = carol.primary_key().key().clone()
 //!     .parts_into_secret()?.into_keypair()?;
@@ -180,8 +180,8 @@
 //!         &mut carol_signer,
 //!         ca.primary_key().component(),
 //!         ca_userid)?;
-//! let ca = ca.insert_packets(ca_tsig.clone())?;
-//! # assert!(ca.clone().into_packets2().any(|p| {
+//! let ca = ca.insert_packets(ca_tsig.clone())?.0;
+//! # assert!(ca.clone().into_packets().any(|p| {
 //! #   match p {
 //! #       Packet::Signature(sig) => sig == ca_tsig,
 //! #       _ => false,
@@ -193,10 +193,10 @@
 //! // using the CA as a trusted introducer based on `ca_tsig`.
 //! let res = RegexSet::from_signature(&ca_tsig)?;
 //!
-//! // Should should be able to authenticate Alice.
+//! // Should be able to authenticate Alice.
 //! let alice_ua = alice.with_policy(p, None)?
 //!     .userids().nth(0).expect("Added a User ID");
-//! # assert!(res.matches_userid(&alice_ua));
+//! # assert!(res.matches_userid(alice_ua.userid()));
 //! let mut authenticated = false;
 //! for c in alice_ua.certifications() {
 //!     if c.get_issuers().into_iter().any(|h| h.aliases(ca.key_handle())) {
@@ -205,7 +205,7 @@
 //!             alice.primary_key().key(),
 //!             alice_ua.userid()).is_ok()
 //!         {
-//!             authenticated |= res.matches_userid(&alice_ua);
+//!             authenticated |= res.matches_userid(alice_ua.userid());
 //!         }
 //!     }
 //! }
@@ -216,7 +216,7 @@
 //! // scope (some.org, not example.com).
 //! let bob_ua = bob.with_policy(p, None)?
 //!     .userids().nth(0).expect("Added a User ID");
-//! # assert!(! res.matches_userid(&bob_ua));
+//! # assert!(! res.matches_userid(bob_ua.userid()));
 //! let mut have_certification = false;
 //! let mut authenticated = false;
 //! for c in bob_ua.certifications() {
@@ -227,7 +227,7 @@
 //!             bob_ua.userid()).is_ok()
 //!         {
 //!             have_certification = true;
-//!             authenticated |= res.matches_userid(&bob_ua);
+//!             authenticated |= res.matches_userid(bob_ua.userid());
 //!         }
 //!     }
 //! }
@@ -355,7 +355,7 @@ fn generate_class(caret: bool, chars: impl Iterator<Item=char>) -> Hir
 /// strings.
 ///
 /// A `Regex` contains a regular expression compiled according to the
-/// rules defined in [Section 8 of RFC 4880] modulo two differences.
+/// rules defined in [Section 8 of RFC 9580] modulo two differences.
 /// First, the compiler only works on UTF-8 strings (not bytes).
 /// Second, ranges in character classes are between UTF-8 characters,
 /// not just ASCII characters.  Further, by default, strings that
@@ -363,8 +363,8 @@ fn generate_class(caret: bool, chars: impl Iterator<Item=char>) -> Hir
 /// characters) never match.  This behavior can be customized using
 /// [`Regex::disable_sanitizations`].
 ///
-///   [Section 8 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-8
-///   [trust signatures]: https://tools.ietf.org/html/rfc4880#section-5.2.3.13
+///   [Section 8 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-8
+///   [trust signatures]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.21
 ///   [`Regex::disable_sanitizations`]: Regex::disable_sanitizations()
 ///
 /// Regular expressions are used to scope the trust that [trust
@@ -423,7 +423,7 @@ impl Regex {
 
         // Converting the Hir to a string and the compiling that is
         // apparently the canonical way to convert a Hir to a Regex
-        // (at least it is what rip-grep does, which the author of
+        // (at least it is what rip-grep does), which the author of
         // regex also wrote.  See
         // ripgrep/crates/regex/src/config.rs:ConfiguredHir::regex.
         let regex = regex::RegexBuilder::new(&hir.to_string())
@@ -523,7 +523,7 @@ assert_send_and_sync!(RegexSet_);
 ///
 /// A `RegexSet` encapsulates a set of regular expressions.  The
 /// regular expressions are compiled according to the rules defined in
-/// [Section 8 of RFC 4880] modulo two differences.  First, the
+/// [Section 8 of RFC 9580] modulo two differences.  First, the
 /// compiler only works on UTF-8 strings (not bytes).  Second, ranges
 /// in character classes are between UTF-8 characters, not just ASCII
 /// characters.  Further, by default, strings that don't pass a sanity
@@ -531,7 +531,7 @@ assert_send_and_sync!(RegexSet_);
 /// match.  This behavior can be customized using
 /// [`RegexSet::disable_sanitizations`].
 ///
-///   [Section 8 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-8
+///   [Section 8 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-8
 ///   [`RegexSet::disable_sanitizations`]: RegexSet::disable_sanitizations()
 ///
 /// `RegexSet` implements the semantics of [regular expression]s used
@@ -542,8 +542,8 @@ assert_send_and_sync!(RegexSet_);
 ///   - Have no Regular Expression subpackets, and/or
 ///   - Include one or more Regular Expression subpackets that are invalid.
 ///
-///   [regular expressions]: https://tools.ietf.org/html/rfc4880#section-5.2.3.14
-///   [Trust Signatures]: https://tools.ietf.org/html/rfc4880#section-5.2.3.13
+///   [regular expressions]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.22
+///   [Trust Signatures]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.21
 ///
 /// `RegexSet` compiles each regular expression individually.  If
 /// there are no regular expressions, the `RegexSet` matches
@@ -806,7 +806,7 @@ impl RegexSet {
     /// regular expressions from a [Trust Signature] and wraps them in a
     /// `RegexSet`.
     ///
-    ///   [Trust Signature]: https://tools.ietf.org/html/rfc4880#section-5.2.3.13
+    ///   [Trust Signature]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3.21
     ///
     /// If the signature is not a valid trust signature (its [type] is
     /// [GenericCertification], [PersonaCertification],
@@ -814,7 +814,7 @@ impl RegexSet {
     /// [Trust Signature] subpacket is present), this returns an
     /// error.
     ///
-    ///   [type]: https://tools.ietf.org/html/rfc4880#section-5.2.1
+    ///   [type]: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.1
     ///   [GenericCertification]: crate::types::SignatureType::GenericCertification
     ///   [PersonaCertification]: crate::types::SignatureType::PersonaCertification
     ///   [CasualCertification]: crate::types::SignatureType::CasualCertification
@@ -841,13 +841,13 @@ impl RegexSet {
     /// # let p = &StandardPolicy::new();
     /// #
     /// # let (alice, _)
-    /// #     = CertBuilder::general_purpose(None, Some("Alice <alice@example.org>"))
+    /// #     = CertBuilder::general_purpose(Some("Alice <alice@example.org>"))
     /// #         .generate()?;
     /// # let mut alices_signer = alice.primary_key().key().clone()
     /// #     .parts_into_secret()?.into_keypair()?;
     /// #
     /// # let (example_com, _)
-    /// #     = CertBuilder::general_purpose(None, Some("OpenPGP CA <openpgp-ca@example.com>"))
+    /// #     = CertBuilder::general_purpose(Some("OpenPGP CA <openpgp-ca@example.com>"))
     /// #         .generate()?;
     /// # let example_com_userid = example_com.with_policy(p, None)?
     /// #     .userids().nth(0).expect("Added a User ID").userid();
@@ -881,7 +881,7 @@ impl RegexSet {
     /// // email addresses wrapped in <>.
     /// assert!(! res.is_match("dave@example.com"));
     ///
-    /// // And, it is case sensitive.
+    /// // And, it is case-sensitive.
     /// assert!(res.is_match("Ellen <ellen@example.com>"));
     /// assert!(! res.is_match("Ellen <ellen@EXAMPLE.COM>"));
     /// # Ok(()) }
@@ -925,13 +925,13 @@ impl RegexSet {
     /// This can be used to optimize the evaluation of scoping rules
     /// along a path: if a `RegexSet` matches everything, then it
     /// doesn't further constrain the path.
-    pub fn everything() -> Result<Self>
+    pub fn everything() -> Self
     {
-        Ok(Self {
+        Self {
             re_bytes: vec![vec![]],
             re_set: RegexSet_::Everything,
             disable_sanitizations: false,
-        })
+        }
     }
 
     /// Returns whether a `RegexSet` matches everything.
@@ -957,7 +957,7 @@ impl RegexSet {
     /// use openpgp::regex::RegexSet;
     ///
     /// # fn main() -> openpgp::Result<()> {
-    /// assert!(RegexSet::everything()?.matches_everything());
+    /// assert!(RegexSet::everything().matches_everything());
     /// let empty: &[ &str ] = &[];
     /// assert!(RegexSet::new(empty)?.matches_everything());
     ///
