@@ -36,21 +36,20 @@ impl Cert {
     /// # fn main() -> openpgp::Result<()> {
     /// // By default, certificates are exportable.
     /// let (cert, _) =
-    ///     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    ///     CertBuilder::general_purpose(Some("alice@example.org"))
     ///         .generate()?;
     /// assert!(cert.exportable());
     ///
     /// // Setting the exportable flag to false makes them
     /// // not-exportable.
     /// let (cert, _) =
-    ///     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    ///     CertBuilder::general_purpose(Some("alice@example.org"))
     ///         .set_exportable(false)
     ///         .generate()?;
     /// assert!(! cert.exportable());
     /// # Ok(())
     /// # }
     /// ```
-    #[allow(clippy::if_same_then_else)]
     pub fn exportable(&self) -> bool {
         let pk = self.primary_key();
 
@@ -95,7 +94,7 @@ impl Cert {
     ///   - Self signatures.  They are authenticated.
     ///   - Other signatures.  They are not authenticated at this point.
     ///   - Other revocations.  They are not authenticated, and likely
-    ///     not well supported in other implementations, hence the
+    ///     not well-supported in other implementations, hence the
     ///     least reliable way of revoking keys and therefore least
     ///     useful and most likely to be abused.
     fn serialize_common(&self, o: &mut dyn std::io::Write, export: bool)
@@ -155,7 +154,7 @@ impl Cert {
             }
         }
 
-        for k in self.subkeys() {
+        for k in self.keys().subkeys() {
             if export && ! k.self_signatures().chain(k.self_revocations()).any(
                 |s| s.exportable().is_ok())
             {
@@ -233,7 +232,7 @@ impl MarshalInto for Cert {
             }
         }
 
-        for k in self.subkeys() {
+        for k in self.keys().subkeys() {
             l += PacketRef::PublicSubkey(k.key()).serialized_len();
 
             for s in k.signatures() {
@@ -313,7 +312,7 @@ impl Cert {
 /// ```
 pub struct TSK<'a> {
     pub(crate) cert: Cow<'a, Cert>,
-    filter: Box<dyn Fn(&key::UnspecifiedSecret) -> bool + 'a>,
+    filter: Box<dyn Fn(&key::UnspecifiedSecret) -> bool + Send + Sync + 'a>,
     emit_stubs: bool,
 }
 
@@ -335,7 +334,7 @@ impl PartialEq for TSK<'_> {
                    && (other.filter)(b.key().parts_as_secret().expect("has_secret")))
             {
                 // Both have secrets.  Compare secrets.
-                (true, true) => if a.optional_secret() != b.optional_secret() {
+                (true, true) => if a.key().optional_secret() != b.key().optional_secret() {
                     return false;
                 },
                 // No secrets.  Equal iff both or neither emit stubs.
@@ -375,7 +374,7 @@ impl<'a> TSK<'a> {
     /// Decomposes the TSK.
     pub(crate) fn decompose(self)
                             -> (Cow<'a, Cert>,
-                                Box<dyn 'a + Fn(&key::UnspecifiedSecret) -> bool>,
+                                Box<dyn Fn(&key::UnspecifiedSecret) -> bool + Send + Sync + 'a>,
                                 bool)
     {
         (self.cert, self.filter, self.emit_stubs)
@@ -418,12 +417,12 @@ impl<'a> TSK<'a> {
     ///     .serialize(&mut buf)?;
     ///
     /// let cert_ = Cert::from_bytes(&buf)?;
-    /// assert!(! cert_.primary_key().has_secret());
+    /// assert!(! cert_.primary_key().key().has_secret());
     /// assert_eq!(cert_.keys().with_policy(p, None).alive().revoked(false).secret().count(), 1);
     /// # Ok(()) }
     /// ```
     pub fn set_filter<P>(mut self, predicate: P) -> Self
-        where P: 'a + Fn(&key::UnspecifiedSecret) -> bool
+        where P: Fn(&key::UnspecifiedSecret)-> bool + Send + Sync + 'a
     {
         self.filter = Box::new(predicate);
         self
@@ -439,7 +438,7 @@ impl<'a> TSK<'a> {
     ///
     /// The default way is to simply emit public key packets when no
     /// secret key material is available.  While straight forward,
-    /// this may be in violation of [Section 11.2 of RFC 4880].
+    /// this may be in violation of [Section 10.2 of RFC 9580].
     ///
     /// The alternative is to emit a secret key packet with a
     /// placeholder secret key value.  GnuPG uses this variant with a
@@ -449,7 +448,7 @@ impl<'a> TSK<'a> {
     /// See [this test] for support in other implementations.
     ///
     ///   [`TSK::set_filter`]: TSK::set_filter()
-    ///   [Section 11.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-11.2
+    ///   [Section 10.2 of RFC 9580]: https://www.rfc-editor.org/rfc/rfc9580.html#section-10.2
     ///   [`S2K`]: super::crypto::S2K
     ///   [this test]: https://tests.sequoia-pgp.org/#Detached_primary_key
     ///
@@ -484,11 +483,11 @@ impl<'a> TSK<'a> {
     /// #            Some(packet::Tag::SecretKey));
     /// let cert_ = Cert::from_bytes(&buf)?;
     /// // The primary key has an "encrypted" stub.
-    /// assert!(cert_.primary_key().has_secret());
+    /// assert!(cert_.primary_key().key().has_secret());
     /// assert_eq!(cert_.keys().with_policy(p, None)
     ///            .alive().revoked(false).unencrypted_secret().count(), 1);
     /// # if let Some(SecretKeyMaterial::Encrypted(sec)) =
-    /// #     cert_.primary_key().optional_secret()
+    /// #     cert_.primary_key().key().optional_secret()
     /// # {
     /// #     assert_eq!(sec.algo(), SymmetricAlgorithm::Unencrypted);
     /// #     if let S2K::Private { tag, .. } = sec.s2k() {
@@ -645,7 +644,7 @@ impl<'a> TSK<'a> {
             }
         }
 
-        for k in self.cert.subkeys() {
+        for k in self.cert.keys().subkeys() {
             if export && ! k.self_signatures().chain(k.self_revocations()).any(
                 |s| s.exportable().is_ok())
             {
@@ -716,8 +715,14 @@ impl<'a> MarshalInto for TSK<'a> {
             if self.emit_stubs && (tag == Tag::PublicKey
                                    || tag == Tag::PublicSubkey) {
                 // Emit a GnuPG-style secret key stub.  The stub
-                // extends the public key by 8 bytes.
-                let l = key.parts_as_public().net_len() + 8;
+                // extends the public key by 8 bytes (plus 4 secret
+                // length octets for v6).
+                let l = key.parts_as_public().net_len()
+                    + match key.version() {
+                        4 => 8,
+                        6 => 12,
+                        _ => 0, // Serialization will fail anyway.
+                    };
                 return 1 // CTB
                     + BodyLength::Full(l as u32).serialized_len()
                     + l;
@@ -764,7 +769,7 @@ impl<'a> MarshalInto for TSK<'a> {
             }
         }
 
-        for k in self.cert.subkeys() {
+        for k in self.cert.keys().subkeys() {
             l += serialized_len_key(k.key().into(),
                                     Tag::PublicSubkey, Tag::SecretSubkey);
 
@@ -883,7 +888,7 @@ mod test {
         use crate::types::{Curve, KeyFlags, SignatureType};
         use crate::packet::{
             signature, UserID, user_attribute::{UserAttribute, Subpacket},
-            key::Key4,
+            key::Key6,
         };
 
         let p = &P::new();
@@ -893,7 +898,7 @@ mod test {
             .unwrap().into_keypair().unwrap();
 
         let key: key::SecretSubkey =
-            Key4::generate_ecc(false, Curve::Cv25519).unwrap().into();
+            Key6::generate_ecc(false, Curve::Cv25519).unwrap().into();
         let key_binding = key.bind(
             &mut keypair, &cert,
             signature::SignatureBuilder::new(SignatureType::SubkeyBinding)
@@ -928,7 +933,7 @@ mod test {
             Packet::SecretSubkey(key), key_binding.into(),
             uid.into(), uid_binding.into(),
             ua.into(), ua_binding.into(),
-        ]).unwrap();
+        ]).unwrap().0;
 
         assert_eq!(cert.subkeys().count(), 1);
         cert.subkeys().next().unwrap().binding_signature(p, None).unwrap();
@@ -1030,7 +1035,7 @@ mod test {
         assert_eq!(cert_.keys().with_policy(p, None)
                    .alive().revoked(false).unencrypted_secret().count(), 1);
         if let Some(SecretKeyMaterial::Encrypted(sec)) =
-            cert_.primary_key().optional_secret()
+            cert_.primary_key().key().optional_secret()
         {
             assert_eq!(sec.algo(), SymmetricAlgorithm::Unencrypted);
             if let S2K::Private { tag, .. } = sec.s2k() {
